//**********************************************************************
//  File name: cddecli.cpp
//
//  Description: 
//            Declares member functions and static data for the 
//            CDDEClient and CDDEClientConv classes
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**********************************************************************

// COPYRIGHT:
//
//   (C) Copyright Microsoft Corp. 1993.  All rights reserved.
//
//   You have a royalty-free right to use, modify, reproduce and
//   distribute the Sample Files (and/or any modified version) in
//   any way you find useful, provided that you agree that
//   Microsoft has no warranty obligations or liability for any
//   Sample Application Files which are modified.
//
                                               
#include <afxwin.h>
#include "cddecli.h"

IMPLEMENT_DYNAMIC( CDDEClient, CObject)
IMPLEMENT_DYNAMIC( CDDEClientConv, CObject)

// static pointer to CDDEClient
CDDEClient * CDDEClient::m_pClient=NULL;

//***********************************************************
//
//  CDDEClient :: CDDEClient()
//
//  Purpose:  Constructor - Initializes pointer and calls 
//            CDDEClient::Initialize() 
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
   CDDEClient::CDDEClient()
   {
   m_pClient = this;
   m_idInst = 0L;
   Initialize();
   }

//***********************************************************
//
//  CDDEClient :: ~CDDEClient()
//
//  Purpose:  Destructor - calls CDDEClient::Uninitialize() 
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
   CDDEClient::~CDDEClient()
   {
   Uninitialize();
   }
   
//***********************************************************
//
//  CDDEClient :: DdeInitialize()
//
//  Purpose:  Initializes client and callback function
//          
//  Parameters:  DWORD afCmd
//               DWORD uRes 
//
//  Return: (UINT)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
UINT CDDEClient::DdeInitialize( DWORD afCmd, DWORD uRes)
{
    m_lpCallback = (PFNCALLBACK)MakeProcInstance( (FARPROC)DDEClientCallback, AfxGetInstanceHandle());
    return(::DdeInitialize(&m_idInst, m_lpCallback, afCmd, uRes));
}

//***********************************************************
//
//  CDDEClient :: DdeUninitialize()
//
//  Purpose:  Uninitializes client and callback function
//          
//  Parameters:
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
BOOL CDDEClient::DdeUninitialize()
{
    BOOL bRet;
    
    bRet = ::DdeUninitialize(m_idInst);
    FreeProcInstance((FARPROC)m_lpCallback);
    return bRet;
}

//***********************************************************
//
//  CDDEClient :: DDEClientCallback()
//
//  Purpose:  Dispatches callback transactions to the appropriate
//            objects.
//          
//  Parameters: UINT type
//              UINT fmt
//              HCONV hConv
//              HSZ hsz1
//              HSZ hsz2
//              HDDEDATA hData
//              DWORD dwData1
//              DWORD dwData2
//
//  Return: (HDDEDATA)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************

HDDEDATA _export CALLBACK CDDEClient::DDEClientCallback( UINT type, UINT fmt,
             HCONV hConv, HSZ hsz1, HSZ hsz2, HDDEDATA hData, DWORD dwData1,
             DWORD dwData2)
{
    // look up the object
    
    CDDEClientConv * pThisConv;
    if(hConv)
       pThisConv = FromHandle(hConv);
    
    switch(type)
       {
       case XTYP_ADVDATA:
          return (pThisConv->OnAdvData(hsz1,hsz2,hData));
       case XTYP_DISCONNECT:
          pThisConv->OnDisconnect((BOOL)dwData2);
          break; 
       case XTYP_REGISTER:
          CDDEClient::m_pClient->OnRegister(hsz1,hsz2);
          break;
       case XTYP_XACT_COMPLETE:
          pThisConv->OnXactComplete(hsz1,hsz2,hData,dwData1,dwData2);
          break;
       case XTYP_UNREGISTER: 
          CDDEClient::m_pClient->OnUnregister(hsz1,hsz2);
          break;
        };
     return NULL; 
}

//***********************************************************
//
//  CDDEClient :: FromHandle()
//
//  Purpose:  Looks up a Client Conversation object based on 
//            its handle.
//          
//  Parameters: HCONV hConv
//
//  Return: (CDDEClientConv *)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
CDDEClientConv * CDDEClient::FromHandle(HCONV hConv)
{
   CDDEClientConv *theConv;
   
   if(m_pClient)
      {
      (m_pClient->m_mapConv).Lookup((DWORD)hConv, (CObject *&)theConv);
      return theConv;
      }
   else
      return NULL;
}    

//***********************************************************
//
//  CDDEClient :: OnRegister()
//
//  Purpose:  Handles XTYP_REGISTER callback transaction
//          
//  Parameters:  HSZ hszBaseServName
//               HSZ hszInstServName
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************

void CDDEClient::OnRegister( HSZ hszBaseServName, HSZ hszInstServName )
{};

//***********************************************************
//
//  CDDEClient :: OnUnregister()
//
//  Purpose:  Handles XTYP_UNREGISTER callback transaction
//          
//  Parameters:  HSZ hszBaseServName
//               HSZ hszInstServName
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CDDEClient::OnUnregister( HSZ hszBaseServName, HSZ hszInstServName )
{};

//***********************************************************
//
//  CDDEClient :: DdeEnableCallback()
//
//  Purpose:  wraps ::DdeEnableCallback()
//          
//  Parameters:  UINT uCmd
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   
BOOL CDDEClient::DdeEnableCallback(UINT uCmd)
   {return ::DdeEnableCallback(m_idInst,NULL,uCmd);}
   
//***********************************************************
//
//  CDDEClient :: DdeGetLastError()
//
//  Purpose:  wraps ::DdeGetLastError()
//          
//  Parameters:  
//
//  Return: (UINT)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************      
UINT CDDEClient::DdeGetLastError()
   {return :: DdeGetLastError(m_idInst);}

//***********************************************************
//
//  CDDEClient :: DdeCreateStringHandle()
//
//  Purpose:  wraps ::DdeCreateStringHandle()
//          
//  Parameters:  LPCSTR lpszString
//               int codepage
//
//  Return: (HSZ)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************      
HSZ CDDEClient::DdeCreateStringHandle(LPCSTR lpszString,int codepage)
   {return ::DdeCreateStringHandle(m_idInst, lpszString, codepage);}
   
//***********************************************************
//
//  CDDEClient :: DdeCreateDataHandle()
//
//  Purpose:  wraps ::DdeCreateDataHandle()
//          
//  Parameters: void FAR * lpvSrcBuf
//              DWORD cbInitData
//              DWORD OffSrcBuf
//              HSZ hszItem
//              UINT uFmt
//              UINT afCmd
//
//  Return: (HDDEDATA)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************           
HDDEDATA CDDEClient::DdeCreateDataHandle(void FAR * lpvSrcBuf,DWORD cbInitData,
            DWORD OffSrcBuf, HSZ hszItem, UINT uFmt, UINT afCmd)
    {return ::DdeCreateDataHandle(m_idInst, lpvSrcBuf, cbInitData, 
       OffSrcBuf, hszItem, uFmt, afCmd);}
       
//***********************************************************
//
//  CDDEClient :: DdeFreeDataHandle()
//
//  Purpose:  wraps ::DdeFreeDataHandle()
//          
//  Parameters: HDDEDATA hData
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************        
BOOL CDDEClient::DdeFreeDataHandle(HDDEDATA hData)
    {return ::DdeFreeDataHandle(hData);}
    
//***********************************************************
//
//  CDDEClient :: DdeFreeStringHandle()
//
//  Purpose:  wraps ::DdeFreeDataHandle()
//          
//  Parameters: HSZ hsz
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************    
BOOL CDDEClient::DdeFreeStringHandle(HSZ hsz)
    {return ::DdeFreeStringHandle(m_idInst, hsz);}
    
//***********************************************************
//
//  CDDEClient :: DdeKeepStringHandle()
//
//  Purpose:  wraps ::DdeKeepDataHandle()
//          
//  Parameters: HSZ hsz
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************    
BOOL CDDEClient::DdeKeepStringHandle(HSZ hsz)
    {return ::DdeKeepStringHandle(m_idInst, hsz);}
    
//***********************************************************
//
//  CDDEClient :: DdeQueryString()
//
//  Purpose:  wraps ::DdeQueryString()
//          
//  Parameters: HSZ hsz
//              LPSTR lpsz
//              DWORD cchMax
//              int codepage
//
//  Return: (DWORD)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
DWORD CDDEClient::DdeQueryString(HSZ hsz, LPSTR lpsz,DWORD cchMax, int codepage)
    {return ::DdeQueryString(m_idInst, hsz, lpsz, cchMax, codepage);}

//***********************************************************
//
//  CDDEClient :: DdeReconnect()
//
//  Purpose:  wraps ::DdeReconnect()
//          
//  Parameters: HCONV hConv
//
//  Return: (HCONV)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************    
HCONV CDDEClient::DdeReconnect(HCONV hConv)
    {return ::DdeReconnect(hConv);}
    
//***********************************************************
//
//  CDDEClient :: DdeUnaccessData()
//
//  Purpose:  wraps ::DdeUnaccessData()
//          
//  Parameters: HDDEDATA hData
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
BOOL CDDEClient::DdeUnaccessData(HDDEDATA hData)
    {return ::DdeUnaccessData(hData);}

    
//***********************************************************
//
//  CDDEClientConv :: CDDEClientConv()
//
//  Purpose:  Constructor - initializes data
//          
//  Parameters: 
//
//  Return: ()
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
CDDEClientConv::CDDEClientConv() 
{ 
   m_hConv = 0;
   m_hszTopic = 0;
   m_pClient = CDDEClient::m_pClient;
   m_hszService = 0;
}

//***********************************************************
//
//  CDDEClientConv :: ~CDDEClientConv()
//
//  Purpose:  Destructor - disconnects conversation
//          
//  Parameters: 
//
//  Return: ()
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
CDDEClientConv::~CDDEClientConv()
{
   DdeDisconnect();
}

//***********************************************************
//
//  CDDEClientConv :: DdeQueryConvInfo()
//
//  Purpose:  wraps ::DdeQueryConvInfo()
//          
//  Parameters: DWORD idTransaction
//              CONVINFO FAR * lpConvInfo 
//
//  Return: (UINT)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   
UINT CDDEClientConv::DdeQueryConvInfo(DWORD idTransaction, CONVINFO FAR * lpConvInfo)
    {return ::DdeQueryConvInfo(m_hConv, idTransaction, lpConvInfo);}
    
//***********************************************************
//
//  CDDEClientConv :: DdeSetUserHandle()
//
//  Purpose:  wraps ::DdeSetUserHandle()
//          
//  Parameters: DWORD id
//              DWORD hUser
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//************************************************************* 
BOOL CDDEClientConv::DdeSetUserHandle(DWORD id, DWORD hUser)
    {return ::DdeSetUserHandle(m_hConv, id, hUser);}    

//***********************************************************
//
//  CDDEClientConv :: DdeEnableCallback()
//
//  Purpose:  wraps ::DdeEnableCallback()
//          
//  Parameters: UINT uCmd
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************    
BOOL CDDEClientConv::DdeEnableCallback(UINT uCmd)
   {return ::DdeEnableCallback(m_pClient->m_idInst,m_hConv,uCmd);}
  
//***********************************************************
//
//  CDDEClientConv :: DdeAbandonTransaction()
//
//  Purpose:  wraps ::DdeAbandonTransaction()
//          
//  Parameters: DWORD dwTransaction
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   
BOOL CDDEClientConv::DdeAbandonTransaction(DWORD dwTransaction)
   {return ::DdeAbandonTransaction(m_pClient->m_idInst, m_hConv,dwTransaction);}

//***********************************************************
//
//  CDDEClientConv :: DdeClientTransaction()
//
//  Purpose:  wraps ::DdeClientTransaction()
//          
//  Parameters: void FAR * lpvData
//              DWORD cbData
//              HSZ hszItem
//              UINT uFmt
//              UINT uType
//              DWORD uTimeout
//              DWORD far *lpuResult
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   
HDDEDATA CDDEClientConv::DdeClientTransaction(void FAR * lpvData, DWORD cbData, HSZ hszItem,
            UINT uFmt, UINT uType, DWORD uTimeout, DWORD far *lpuResult)
   {return ::DdeClientTransaction(lpvData, cbData, m_hConv, hszItem,
                uFmt, uType, uTimeout, lpuResult); }

//***********************************************************
//
//  CDDEClientConv :: DdeConnect()
//
//  Purpose:  wraps ::DdeConnect()
//          
//  Parameters: LPCSTR lpcszService
//              LPCSTR lpcszTopic
//              CONVCONTEXT FAR *lpcc
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************                   
BOOL CDDEClientConv::DdeConnect(  LPCSTR lpcszService, LPCSTR lpcszTopic,CONVCONTEXT FAR *lpcc)
   { 
   m_hszTopic = m_pClient->DdeCreateStringHandle(lpcszTopic);
   m_hszService = m_pClient->DdeCreateStringHandle(lpcszService);
   m_hConv = ::DdeConnect(m_pClient->m_idInst, m_hszService, m_hszTopic, lpcc);
     
   // add conversation to map 
   m_pClient->m_mapConv.SetAt((DWORD)m_hConv, (CObject *)this);  
   
   return (BOOL)m_hConv;
   }
   
//***********************************************************
//
//  CDDEClientConv :: DdeDisconnect()
//
//  Purpose:  wraps ::DdeDisconnect()
//          
//  Parameters:
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   
BOOL CDDEClientConv::DdeDisconnect()
   {
   BOOL b = TRUE;
   
   if(m_hConv)
      {
      b = ::DdeDisconnect(m_hConv);
      m_pClient->DdeFreeStringHandle( m_hszTopic);
      m_pClient->DdeFreeStringHandle( m_hszService);
   
      // remove conversation from map
      m_pClient->m_mapConv.RemoveKey((DWORD)m_hConv);
      }
   return b;
   }         
  
//***********************************************************
//
//  CDDEClientConv :: OnAdvData()
//
//  Purpose:  Handles XTYP_ADVDATA transaction
//          
//  Parameters: HSZ hszTopic
//              HSZ hszItem
//              HDDEDATA hDataAdvise
//
//  Return: (HDDEDATA)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   
HDDEDATA CDDEClientConv::OnAdvData( HSZ hszTopic, HSZ hszItem, HDDEDATA hDataAdvise)
{return NULL;};

//***********************************************************
//
//  CDDEClientConv :: OnDisconnect()
//
//  Purpose:  Handles XTYP_DISCONNECT transaction
//          
//  Parameters: BOOL fSameInst
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   

void CDDEClientConv::OnDisconnect( BOOL fSameInst)
  {
  m_pClient->m_mapConv.RemoveKey((DWORD)m_hConv);
  m_hConv = 0;
  m_pClient->DdeFreeStringHandle(m_hszTopic);
  m_pClient->DdeFreeStringHandle( m_hszService);
  }

//***********************************************************
//
//  CDDEClientConv :: OnXactComplete()
//
//  Purpose:  Handles XTYP_XACT_COMPLETE transaction
//          
//  Parameters: HSZ hszTopic
//              HSZ hszItem
//              HDDEDATA hDataXact
//              DWORD dwXactID
//              DWORD fwStatus
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   
void CDDEClientConv::OnXactComplete( HSZ hszTopic, HSZ hszItem, HDDEDATA hDataXact,
                                DWORD dwXactID, DWORD fwStatus)
{};

//***********************************************************
//
//  CDDEClientConv :: DdePostAdvise()
//
//  Purpose:  wraps ::DdePostAdvise
//          
//  Parameters: HSZ hszItem
//            
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
BOOL CDDEClientConv::DdePostAdvise(HSZ hszItem)
    {return ::DdePostAdvise(m_pClient->m_idInst, m_hszTopic, hszItem);}
    

    
