//**********************************************************************
//  File name: cddeserv.cpp
//
//  Description: 
//            Declares member functions and static data for the 
//            CDDEServer and CDDEServerConv classes
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**********************************************************************

// COPYRIGHT:
//
//   (C) Copyright Microsoft Corp. 1993.  All rights reserved.
//
//   You have a royalty-free right to use, modify, reproduce and
//   distribute the Sample Files (and/or any modified version) in
//   any way you find useful, provided that you agree that
//   Microsoft has no warranty obligations or liability for any
//   Sample Application Files which are modified.
//                                                          

#include <afxwin.h>
#include "cddeserv.h"

IMPLEMENT_DYNAMIC(CDDEServer, CObject)
IMPLEMENT_DYNAMIC(CDDEServerConv, CObject)

CDDEServer * CDDEServer::m_pServer=NULL;

//***********************************************************
//
//  CDDEServer :: CDDEServer()
//
//  Purpose:  Constructor - Initializes pointer and calls 
//            CDDEServer::Initialize() 
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
   CDDEServer::CDDEServer()
   {
   m_pServer = this;
   m_idInst = 0L;
   Initialize();
   }

//***********************************************************
//
//  CDDEServer :: ~CDDEServer()
//
//  Purpose:  Destructor - calls CDDEServer::Uninitialize() 
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
 
   CDDEServer::~CDDEServer()
   {
   Uninitialize();
   }
   
   

//***********************************************************
//
//  CDDEServer :: DdeInitialize()
//
//  Purpose:  Initializes Server and callback function
//          
//  Parameters:  DWORD afCmd
//               DWORD uRes 
//
//  Return: (UINT)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************


UINT CDDEServer::DdeInitialize( DWORD afCmd, DWORD uRes)
{
    m_lpCallback = (PFNCALLBACK)MakeProcInstance( (FARPROC)DDEServerCallback, AfxGetInstanceHandle());
    return(::DdeInitialize(&m_idInst, m_lpCallback, afCmd, uRes));
}

//***********************************************************
//
//  CDDEServer :: DdeUninitialize()
//
//  Purpose:  Uninitializes server and callback function
//          
//  Parameters:
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************

BOOL CDDEServer::DdeUninitialize()
{
    BOOL bRet;
    
    bRet = ::DdeUninitialize(m_idInst);
    FreeProcInstance((FARPROC)m_lpCallback);
    return bRet;
}

//***********************************************************
//
//  CDDEServer :: DdeNameService()
//
//  Purpose: wraps ::DdeNameService() 
//          
//  Parameters: HSZ hszServ
//              HSZ hszRes
//              UINT afCmd
//
//  Return: (HDDEDATA)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************

HDDEDATA CDDEServer::DdeNameService(HSZ hszServ, HSZ hszRes, UINT afCmd)
{
   return ::DdeNameService(m_idInst, hszServ,hszRes,afCmd);
}   

//***********************************************************
//
//  CDDEServer :: DDEServerCallback()
//
//  Purpose:  Dispatches callback transactions to the appropriate
//            objects.
//          
//  Parameters: UINT type
//              UINT fmt
//              HCONV hConv
//              HSZ hsz1
//              HSZ hsz2
//              HDDEDATA hData
//              DWORD dwData1
//              DWORD dwData2
//
//  Return: (HDDEDATA)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************


HDDEDATA _export CALLBACK CDDEServer::DDEServerCallback( UINT type, UINT fmt,
             HCONV hConv, HSZ hsz1, HSZ hsz2, HDDEDATA hData, DWORD dwData1,
             DWORD dwData2)
{
    // look up the object
    CDDEServerConv * pThisConv;
    
    if(hConv && (type != XTYP_CONNECT_CONFIRM))
       pThisConv = FromHandle(hConv);
    
    switch(type)
       {
       case XTYP_DISCONNECT:
          pThisConv->OnDisconnect((BOOL)dwData2);
          break; 
       case XTYP_REGISTER:
          CDDEServer::m_pServer->OnRegister(hsz1,hsz2);
          break;
       case XTYP_UNREGISTER: 
          CDDEServer::m_pServer->OnUnregister(hsz1,hsz2);
          break;
       case XTYP_CONNECT:
          return (HDDEDATA) CDDEServer::m_pServer->OnConnect(hsz1,hsz2,(CONVCONTEXT FAR *)dwData1,(BOOL)dwData2);
          break;
       case XTYP_WILDCONNECT:
          return (HDDEDATA) CDDEServer::m_pServer->OnWildConnect(fmt,hsz1,hsz2,(CONVCONTEXT FAR *)dwData1,(BOOL)dwData2);
          break;
       case XTYP_CONNECT_CONFIRM:
          CDDEServer::m_pServer->OnConnectConfirm(hConv,hsz1,hsz2,(BOOL)dwData2);
          break;
       case XTYP_REQUEST:
          return (HDDEDATA) pThisConv->OnRequest(fmt,hsz1,hsz2);
          break;
       case XTYP_ADVREQ:
          return (HDDEDATA) pThisConv->OnAdvReq(fmt,hsz1,hsz2,LOWORD(dwData1));
          break;
       case XTYP_POKE:
          return (HDDEDATA) pThisConv->OnPoke(hsz1,hsz2,hData);
          break;
       case XTYP_EXECUTE:
          return (HDDEDATA) pThisConv->OnExecute(hsz1,hData);
          break;
       case XTYP_ADVSTART:
          return (HDDEDATA) pThisConv->OnAdvStart(hsz2,hsz2);
          break;
       case XTYP_ADVSTOP:
          pThisConv->OnAdvStop(hsz1,hsz2);
          break;
       
        };
     return NULL; 
     
}

//***********************************************************
//
//  CDDEServer :: FromHandle()
//
//  Purpose:  Looks up a Server Conversation object based on 
//            its handle.
//          
//  Parameters: HCONV hConv
//
//  Return: (CDDEServerConv *)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************

CDDEServerConv * CDDEServer::FromHandle(HCONV hConv)
{
   CDDEServerConv *theConv;
   
   if(CDDEServer::m_pServer)
      {
      ((CDDEServer::m_pServer)->m_mapConv).Lookup((DWORD)hConv, (CObject *&)theConv);
      return theConv;
      }
   else
      return NULL;
}

//***********************************************************
//
//  CDDEServer :: OnRegister()
//
//  Purpose:  Handles XTYP_REGISTER callback transaction
//          
//  Parameters:  HSZ hszBaseServName
//               HSZ hszInstServName
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************

void CDDEServer::OnRegister( HSZ hszBaseServName, HSZ hszInstServName )
{
   DdeKeepStringHandle(hszBaseServName);
   DdeKeepStringHandle(hszInstServName); 
       
};

//***********************************************************
//
//  CDDEServer :: OnUnregister()
//
//  Purpose:  Handles XTYP_UNREGISTER callback transaction
//          
//  Parameters:  HSZ hszBaseServName
//               HSZ hszInstServName
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************

void CDDEServer::OnUnregister( HSZ hszBaseServName, HSZ hszInstServName )
{
   DdeFreeStringHandle(hszBaseServName);
   DdeFreeStringHandle(hszInstServName);
};

//***********************************************************
//
//  CDDEServer :: OnConnect()
//
//  Purpose:  Handles XTYP_CONNECT callback transaction
//          
//  Parameters: HSZ hszTopic
//              HSZ hszService
//              CONVCONTEXT FAR *pcc
//              BOOL fSameInstance 
//              
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************

BOOL CDDEServer::OnConnect(HSZ hszTopic,HSZ hszService,CONVCONTEXT FAR *pcc,BOOL fSameInstance)
{
   return FALSE;
}
//***********************************************************
//
//  CDDEServer :: OnWildConnect()
//
//  Purpose:  Handles XYP_WILDCONNECT callback transaction
//                     
//  Parameters: UINT fmt
//              HSZ hszTopic
//              HSZ hszService
//              CONVCONTEXT FAR *pcc
//              BOOL fSameInstance 
//              
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************

HDDEDATA CDDEServer::OnWildConnect(UINT fmt,HSZ hszTopic,HSZ hszService,CONVCONTEXT FAR *pcc,BOOL fSameInstance)
{
   return NULL;
}

//***********************************************************
//
//  CDDEServer :: OnConnectConfirmt()
//
//  Purpose:  Handles XYP_CONNECT_CONFIRM callback transaction
//                     
//  Parameters: HCONV hConv
//              HSZ hszTopic
//              HSZ hszService
//              BOOL fSameInstance 
//              
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************

void CDDEServer::OnConnectConfirm(HCONV hConv,HSZ hszTopic,HSZ hszService,BOOL fSameInstance)
{
}
 
//***********************************************************
//
//  CDDEServer :: DdePostAdvise()
//
//  Purpose:  wraps ::DdePostAdviose()
//          
//  Parameters:  HSZ hszTopic
//               HSZ hszItem
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   
  
BOOL CDDEServer::DdePostAdvise(HSZ hszTopic, HSZ hszItem)
{
   return ::DdePostAdvise(m_idInst, hszTopic, hszItem);
}   

//***********************************************************
//
//  CDDEServer :: DdeEnableCallback()
//
//  Purpose:  wraps ::DdeEnableCallback()
//          
//  Parameters:  UINT uCmd
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   

BOOL CDDEServer::DdeEnableCallback(UINT uCmd)
   {return ::DdeEnableCallback(m_idInst,NULL,uCmd);}

//***********************************************************
//
//  CDDEServer :: DdeGetLastError()
//
//  Purpose:  wraps ::DdeGetLastError()
//          
//  Parameters:  
//
//  Return: (UINT)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************      
   
UINT CDDEServer::DdeGetLastError()
   {return :: DdeGetLastError(m_idInst);}

//***********************************************************
//
//  CDDEServer :: DdeCreateStringHandle()
//
//  Purpose:  wraps ::DdeCreateStringHandle()
//          
//  Parameters:  LPCSTR lpszString
//               int codepage
//
//  Return: (HSZ)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************      
   
HSZ CDDEServer::DdeCreateStringHandle(LPCSTR lpszString,int codepage)
   {return ::DdeCreateStringHandle(m_idInst, lpszString, codepage);}

//***********************************************************
//
//  CDDEServer :: DdeCreateDataHandle()
//
//  Purpose:  wraps ::DdeCreateDataHandle()
//          
//  Parameters: void FAR * lpvSrcBuf
//              DWORD cbInitData
//              DWORD OffSrcBuf
//              HSZ hszItem
//              UINT uFmt
//              UINT afCmd
//
//  Return: (HDDEDATA)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************           
   
HDDEDATA CDDEServer::DdeCreateDataHandle(void FAR * lpvSrcBuf,DWORD cbInitData,
            DWORD OffSrcBuf, HSZ hszItem, UINT uFmt, UINT afCmd)
    {return ::DdeCreateDataHandle(m_idInst, lpvSrcBuf, cbInitData, 
       OffSrcBuf, hszItem, uFmt, afCmd);}

//***********************************************************
//
//  CDDEServer :: DdeFreeDataHandle()
//
//  Purpose:  wraps ::DdeFreeDataHandle()
//          
//  Parameters: HDDEDATA hData
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************        
       
BOOL CDDEServer::DdeFreeDataHandle(HDDEDATA hData)
    {return ::DdeFreeDataHandle(hData);}

//***********************************************************
//
//  CDDEServer :: DdeFreeStringHandle()
//
//  Purpose:  wraps ::DdeFreeDataHandle()
//          
//  Parameters: HSZ hsz
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************    
    
BOOL CDDEServer::DdeFreeStringHandle(HSZ hsz)
    {return ::DdeFreeStringHandle(m_idInst, hsz);}

//***********************************************************
//
//  CDDEServer :: DdeKeepStringHandle()
//
//  Purpose:  wraps ::DdeKeepDataHandle()
//          
//  Parameters: HSZ hsz
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************    
    
BOOL CDDEServer::DdeKeepStringHandle(HSZ hsz)
    {return ::DdeKeepStringHandle(m_idInst, hsz);}
    
//***********************************************************
//
//  CDDEServer :: DdeQueryString()
//
//  Purpose:  wraps ::DdeQueryString()
//          
//  Parameters: HSZ hsz
//              LPSTR lpsz
//              DWORD cchMax
//              int codepage
//
//  Return: (DWORD)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************

DWORD CDDEServer::DdeQueryString(HSZ hsz, LPSTR lpsz,DWORD cchMax, int codepage)
    {return ::DdeQueryString(m_idInst, hsz, lpsz, cchMax, codepage);}

//***********************************************************
//
//  CDDEServer :: DdeReconnect()
//
//  Purpose:  wraps ::DdeReconnect()
//          
//  Parameters: HCONV hConv
//
//  Return: (HCONV)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************    
    
HCONV CDDEServer::DdeReconnect(HCONV hConv)
    {return ::DdeReconnect(hConv);}

//***********************************************************
//
//  CDDEServer :: DdeUnaccessData()
//
//  Purpose:  wraps ::DdeUnaccessData()
//          
//  Parameters: HDDEDATA hData
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
    
BOOL CDDEServer::DdeUnaccessData(HDDEDATA hData)
    {return ::DdeUnaccessData(hData);}
    
//***********************************************************
//
//  CDDEServerConv :: CDDEServerConv()
//
//  Purpose:  Constructor - initializes data
//          
//  Parameters: 
//
//  Return: ()
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************


CDDEServerConv::CDDEServerConv() 
{ 
   m_hConv = 0;
   m_hszTopic = 0;
   m_pServer = CDDEServer::m_pServer;
   m_hszService = 0;
}

//***********************************************************
//
//  CDDEServerConv :: ~CDDEServerConv()
//
//  Purpose:  Destructor - disconnects conversation
//          
//  Parameters: 
//
//  Return: ()
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************

CDDEServerConv::~CDDEServerConv()
{
   DdeDisconnect();
}

//***********************************************************
//
//  CDDEServerConv :: Create()
//
//  Purpose:  Initializes a new conversation object
//          
//  Parameters: HCONV hConv
//              HSZ hszTopic
//              HSZ hszService
//              BOOL fSameInstance
//
//  Return: (UINT)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   

void CDDEServerConv::Create(HCONV hConv, HSZ hszTopic,HSZ hszService, BOOL fSameInstance)
{
   m_hszTopic = hszTopic;
   m_hszService = hszService;
   
   m_pServer->DdeKeepStringHandle(hszTopic);
   m_pServer->DdeKeepStringHandle(hszService);
   m_hConv = hConv;
   
   // add conversation to map 
   m_pServer->m_mapConv.SetAt((DWORD)hConv, (CObject *)this);  
   
}


//***********************************************************
//
//  CDDEServerConv :: DdeQueryConvInfo()
//
//  Purpose:  wraps ::DdeQueryConvInfo()
//          
//  Parameters: DWORD idTransaction
//              CONVINFO FAR * lpConvInfo 
//
//  Return: (UINT)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   
    
UINT CDDEServerConv::DdeQueryConvInfo(DWORD idTransaction, CONVINFO FAR * lpConvInfo)
    {return ::DdeQueryConvInfo(m_hConv, idTransaction, lpConvInfo);}
    
//***********************************************************
//
//  CDDEServerConv :: DdeSetUserHandle()
//
//  Purpose:  wraps ::DdeSetUserHandle()
//          
//  Parameters: DWORD id
//              DWORD hUser
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//************************************************************* 

BOOL CDDEServerConv::DdeSetUserHandle(DWORD id, DWORD hUser)
    {return ::DdeSetUserHandle(m_hConv, id, hUser);}    

//***********************************************************
//
//  CDDEServerConv :: DdeEnableCallback()
//
//  Purpose:  wraps ::DdeEnableCallback()
//          
//  Parameters: UINT uCmd
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************    
    
BOOL CDDEServerConv::DdeEnableCallback(UINT uCmd)
   {return ::DdeEnableCallback(m_pServer->m_idInst,m_hConv,uCmd);}  

//***********************************************************
//
//  CDDEServerConv :: DdeDisconnect()
//
//  Purpose:  wraps ::DdeDisconnect()
//          
//  Parameters:
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   
   
BOOL CDDEServerConv::DdeDisconnect()
   {
   BOOL b = TRUE;
   
   if(m_hConv)
      {
      b = ::DdeDisconnect(m_hConv);
      m_pServer->DdeFreeStringHandle( m_hszTopic);
      m_pServer->DdeFreeStringHandle( m_hszService);
   
      // remove conversation from map
      m_pServer->m_mapConv.RemoveKey((DWORD)m_hConv);
      }
   return b;
   }         

//***********************************************************
//
//  CDDEServerConv :: OnAdvData()
//
//  Purpose:  Handles XTYP_ADVDATA transaction
//          
//  Parameters: HSZ hszTopic
//              HSZ hszItem
//              HDDEDATA hDataAdvise
//
//  Return: (HDDEDATA)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   
   
UINT CDDEServerConv::OnAdvData( HSZ hszTopic, HSZ hszItem, HDDEDATA hDataAdvise)
{return NULL;};

//***********************************************************
//
//  CDDEServerConv :: OnDisconnect()
//
//  Purpose:  Handles XTYP_DISCONNECT transaction
//          
//  Parameters: BOOL fSameInst
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   


void CDDEServerConv::OnDisconnect( BOOL fSameInst)
  {
  m_pServer->m_mapConv.RemoveKey((DWORD)m_hConv);
  m_hConv = 0;
  m_pServer->DdeFreeStringHandle(m_hszTopic);
  m_pServer->DdeFreeStringHandle( m_hszService);
  
  delete this;
  }
  

//***********************************************************
//
//  CDDEServerConv :: OnRequest()
//
//  Purpose:  Handles XTYP_REQUEST transaction
//          
//  Parameters: BOOL fSameInst
//
//  Return: (HDDEDATA)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   

HDDEDATA CDDEServerConv::OnRequest(UINT fmt,HSZ hszTopic,HSZ hszItem)
{
return NULL;
}

//***********************************************************
//
//  CDDEServerConv :: OnAdvReq()
//
//  Purpose:  Handles XTYP_ADVREQ transaction
//          
//  Parameters: UINT wFmt
//              HSZ hszTopic
//              HSZ hszItem
//              UINT cAdvReq
//
//  Return: (HDDEDATA)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   

HDDEDATA CDDEServerConv::OnAdvReq(UINT wFmt,HSZ hszTopic,HSZ hszItem, UINT cAdvReq)
{
return NULL;
}

//***********************************************************
//
//  CDDEServerConv :: OnAdvStart()
//
//  Purpose:  Handles XTYP_ADVSTART transaction
//          
//  Parameters: HSZ hszTopic
//              HSZ hszItem
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   

BOOL CDDEServerConv::OnAdvStart(HSZ hszTopic,HSZ hszItem)
{
return FALSE;
}

//***********************************************************
//
//  CDDEServerConv :: OnAdvStop()
//
//  Purpose:  Handles XTYP_ADVSTOP transaction
//          
//  Parameters: HSZ hszTopic
//              HSZ hszItem
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   

void CDDEServerConv::OnAdvStop(HSZ hszTopic,HSZ hszItem)
{}

//***********************************************************
//
//  CDDEServerConv :: OnPoke()
//
//  Purpose:  Handles XTYP_POKE transaction
//          
//  Parameters: HSZ hszTopic
//              HSZ hszItem
//              HDDEDATA hData
//
//  Return: (UINT)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   

UINT CDDEServerConv::OnPoke(HSZ hszTopic,HSZ hszItem, HDDEDATA hData)
{
return DDE_FNOTPROCESSED;
}

//***********************************************************
//
//  CDDEServerConv :: OnExecute()
//
//  Purpose:  Handles XTYP_EXECUTE transaction
//          
//  Parameters: HSZ hszTopic
//              HDDEDATA hData
//
//  Return: (UINT)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************   


UINT CDDEServerConv::OnExecute(HSZ hszTopic, HDDEDATA hData)
{
return DDE_FNOTPROCESSED;
}



