//**********************************************************************
//  File name: client.cpp
//
//  Description: 
//            Declares member functions and static data for the 
//            CMyClient,  CMainWindow, and CTheApp classes
//            used by the CLIENT application.
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**********************************************************************

// COPYRIGHT:
//
//   (C) Copyright Microsoft Corp. 1993.  All rights reserved.
//
//   You have a royalty-free right to use, modify, reproduce and
//   distribute the Sample Files (and/or any modified version) in
//   any way you find useful, provided that you agree that
//   Microsoft has no warranty obligations or liability for any
//   Sample Application Files which are modified.
//

#include <afxwin.h>
#include "resource.h"

#include "client.h"

// Declares and runs the application
CTheApp theApp;

//***********************************************************
//
//  CMainWindow :: CMainWindow()
//
//  Purpose:  Constructor - Loads accelerator table and creates
//            main window 
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
CMainWindow::CMainWindow()
{
        LoadAccelTable( "MainAccelTable" );
        Create( NULL, "CDDEMLClient Sample",
                WS_OVERLAPPEDWINDOW, rectDefault, NULL, "MainMenu" );
}

//***********************************************************
//
//  CMainWindow :: OnPaint()
//
//  Purpose:  Paints the main window.
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMainWindow::OnPaint()
{
        CPaintDC dc( this );
        if(myConv.myBitmap.m_hObject)
        {
        CDC MemDC;
        CRect rect;
        CBitmap *pOldBitmap;

        MemDC.CreateCompatibleDC(&dc);
        pOldBitmap = MemDC.SelectObject(&(myConv.myBitmap));
        GetClientRect( rect );
        BITMAP bm;
        ::GetObject(myConv.myBitmap.GetSafeHandle(),sizeof(BITMAP),&bm); 
        dc.StretchBlt(0,0,rect.Width(),rect.Height(),&MemDC,
                      0,0,bm.bmWidth,bm.bmHeight,SRCCOPY);
        MemDC.SelectObject(pOldBitmap);
        }              
}

//***********************************************************
//
//  CMainWindow :: OnAbout()
//
//  Purpose:  Brings up the about dialog box.
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMainWindow::OnAbout()
{
        CModalDialog about( "AboutBox", this );
        about.DoModal();
}

//***********************************************************
//
//  CMainWindow :: OnWhatDoIDo()
//
//  Purpose:  Brings up the "what do I do" dialog box.
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMainWindow::OnWhatDoIDo()
{
        CModalDialog what( "WhatDoIDo", this );
        what.DoModal();
}

//***********************************************************
//
//  CMainWindow :: OnStartWarm()
//
//  Purpose:  Starts a warm advise loop
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMainWindow::OnStartWarm()
{
       myConv.DdeClientTransaction(NULL, 0, hszItem, CF_BITMAP, 
            XTYP_ADVSTART | XTYPF_NODATA, 1000);
       
        CMenu *pMenu = GetMenu();

                  
       pMenu->EnableMenuItem( IDM_STARTHOTADVISE,
          MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);

       pMenu->EnableMenuItem( IDM_STARTWARMADVISE,
          MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);

       pMenu->EnableMenuItem( IDM_STOPWARMADVISE,
          MF_BYCOMMAND | MF_ENABLED);


       
}

//***********************************************************
//
//  CMainWindow :: OnStartHot()
//
//  Purpose:  Starts a hot advise loop
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMainWindow::OnStartHot()
{
       myConv.DdeClientTransaction(NULL, 0, hszItem, CF_BITMAP, 
            XTYP_ADVSTART, 1000);
       CMenu *pMenu = GetMenu();
                  
       pMenu->EnableMenuItem( IDM_STARTHOTADVISE, 
                MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);

        pMenu->EnableMenuItem( IDM_STARTWARMADVISE, 
                MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);

        pMenu->EnableMenuItem( IDM_STOPHOTADVISE,
                MF_BYCOMMAND | MF_ENABLED);

       
}

//***********************************************************
//
//  CMainWindow :: OnStopAdv()
//
//  Purpose:  Stops a warm or hot advise loop
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMainWindow::OnStopAdv()
{
       myConv.DdeClientTransaction(NULL, 0, hszItem, CF_BITMAP, 
            XTYP_ADVSTOP, 1000);
      
       CMenu *pMenu = GetMenu();
       
       pMenu->EnableMenuItem( IDM_STOPHOTADVISE,
           MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);

       pMenu->EnableMenuItem( IDM_STARTHOTADVISE,
           MF_BYCOMMAND | MF_ENABLED);

       pMenu->EnableMenuItem( IDM_STOPWARMADVISE,
           MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);

       pMenu->EnableMenuItem( IDM_STARTWARMADVISE,
           MF_BYCOMMAND | MF_ENABLED);
     
      
}

//***********************************************************
//
//  CMainWindow :: OnStartConv()
//
//  Purpose:  Starts a conversation
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMainWindow::OnStartConv()
{
 myConv.DdeConnect("DDEMLDemo","Sample");  
 
 CMenu *pMenu = GetMenu();

 pMenu->EnableMenuItem( IDM_DOTRANSACTION, MF_BYCOMMAND | MF_ENABLED);

 pMenu->EnableMenuItem( IDM_STARTHOTADVISE, MF_BYCOMMAND | MF_ENABLED);

 pMenu->EnableMenuItem( IDM_STARTWARMADVISE, MF_BYCOMMAND | MF_ENABLED);

 pMenu->EnableMenuItem( IDM_CONNECTTOSERVER,  MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);   
}

//***********************************************************
//
//  CMainWindow :: OnCreate()
//
//  Purpose:  Initializes window as it is created
//          
//  Parameters: LPCREATESTRUCT lpc  
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
int CMainWindow::OnCreate(LPCREATESTRUCT lpc)
{
    // initialize menu items
    CMenu  *pMenu = GetMenu();
    
           pMenu->EnableMenuItem( IDM_DOTRANSACTION,
                          MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
           
           pMenu->EnableMenuItem( IDM_STARTHOTADVISE,
                          MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);

           pMenu->EnableMenuItem( IDM_STARTWARMADVISE,
                          MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);

           pMenu->EnableMenuItem( IDM_STOPHOTADVISE,
                          MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);

           pMenu->EnableMenuItem( IDM_STOPWARMADVISE,
                          MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
    
    return 0;    
}

//***********************************************************
//
//  CMainWindow :: OnSysTopics()
//                OnItemList()
//                OnSysItems()
//                OnFormats()
//                OnStatus()
//                OnRtnMsg()
//                OnSvrHelp()
//
//  Purpose:  Displays system information about server
//          
//  Parameters:  
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMainWindow::OnSysTopics()
{
   GetSysConvData(SZDDESYS_ITEM_TOPICS);
}

void CMainWindow::OnItemList()
{
   GetSysConvData(SZDDE_ITEM_ITEMLIST);
}

void CMainWindow::OnSysItems()
{
   GetSysConvData(SZDDESYS_ITEM_SYSITEMS);
}

void CMainWindow::OnFormats()
{
   GetSysConvData(SZDDESYS_ITEM_FORMATS);
}

void CMainWindow::OnStatus()
{
   GetSysConvData(SZDDESYS_ITEM_STATUS);
}

void CMainWindow::OnRtnMsg()
{
   GetSysConvData(SZDDESYS_ITEM_RTNMSG);
}

void CMainWindow::OnSvrHelp()
{
   GetSysConvData(SZDDESYS_ITEM_HELP);
}

//***********************************************************
//
//  CMainWindow ::GetSysConvData()
//
//  Purpose:  Displays information about server
//          
//  Parameters: LPSTR lpszMsg 
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMainWindow::GetSysConvData(LPSTR lpszMsg)
{
  
   HSZ hszItem = (sysConv.m_pClient)->DdeCreateStringHandle(lpszMsg);

   // is there a system conversation already established
   if(!sysConv.m_hConv)
      {
      sysConv.DdeConnect("DDEMLDemo", SZDDESYS_TOPIC);
      }
      
   HDDEDATA hData = sysConv.DdeClientTransaction(NULL,0,hszItem, 
           CF_TEXT, XTYP_REQUEST,1000);    
   
   static char szBuffer[256];
   static char szOut[512];
   
   DWORD dwChars; 
   
   if(!hData)
      lstrcpy(szBuffer,"Error");
   
   else
      {
      dwChars = DdeGetData(hData,NULL,256,0);
      dwChars = DdeGetData(hData,szBuffer,dwChars,0);
      }
   
   (sysConv.m_pClient)->DdeFreeStringHandle(hszItem);
   
   szOut[0] = '\0';
   lstrcat(szOut,lpszMsg);
   lstrcat(szOut,": ");
   lstrcat(szOut,szBuffer);
   
   MessageBox(szOut, "CDDEML Sample",MB_OK | MB_ICONEXCLAMATION);
}
      
// Message map for CMainWindow

BEGIN_MESSAGE_MAP( CMainWindow, CFrameWnd )
        ON_WM_PAINT()
        ON_COMMAND( IDM_ABOUT, OnAbout )
        ON_COMMAND(IDM_WHATDOIDO, OnWhatDoIDo)
        ON_COMMAND(IDM_STARTHOTADVISE, OnStartHot)
        ON_COMMAND(IDM_STARTWARMADVISE, OnStartWarm)
        ON_COMMAND(IDM_STOPHOTADVISE, OnStopAdv)
        ON_COMMAND(IDM_STOPWARMADVISE, OnStopAdv)
        ON_COMMAND(IDM_CONNECTTOSERVER, OnStartConv) 
        ON_COMMAND(IDM_SYSTOPICS,OnSysTopics)
        ON_COMMAND(IDM_ITEMLIST,OnItemList)
        ON_COMMAND(IDM_SYSITEMS,OnSysItems)
        ON_COMMAND(IDM_FORMATS,OnFormats)
        ON_COMMAND(IDM_STATUS,OnStatus)
        ON_COMMAND(IDM_RTNMSG,OnRtnMsg)
        ON_COMMAND(IDM_SVRHELP,OnSvrHelp) 
        ON_WM_CREATE()
END_MESSAGE_MAP()

//***********************************************************
//
//  CTheApp :: InitInstance()
//
//  Purpose:  Initializes each instance of the application
//          
//  Parameters:  
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
BOOL CTheApp::InitInstance()
{   
        m_pDDEClient = new CDDEClient;

        m_pMainWnd = new CMainWindow();
        m_pMainWnd->ShowWindow( m_nCmdShow );
        m_pMainWnd->UpdateWindow();
        ((CMainWindow *)m_pMainWnd)->hszItem = 
             m_pDDEClient->DdeCreateStringHandle("ChangeBitmap");
        ((CMainWindow *)m_pMainWnd)->myConv.pMainWnd = (CMainWindow *)m_pMainWnd;

        return TRUE;
}

//***********************************************************
//
//  CTheApp :: ExitInstance()
//
//  Purpose:  Cleans up after each instance of the application
//          
//  Parameters:  
//
//  Return: (int)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
int CTheApp::ExitInstance()
{
   m_pDDEClient->DdeFreeStringHandle( ((CMainWindow *)m_pMainWnd)->hszItem); 
   delete m_pDDEClient;
   return 0;
}


//***********************************************************
//
//  CMyConv :: OnAdvData()
//
//  Purpose:  Handles XTYP_ADVDATA transcation.  Receives the
//            bitmap and stores it in the CBitmap data member
//            of the conversation.  Then invalidates the main
//            window so that the bitmap will get drawn.
//          
//  Parameters:  
//
//  Return: (int)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
HDDEDATA CMyConv::OnAdvData(HSZ hszTopic,HSZ hszItem, HDDEDATA hDataAdvise)
{
    BOOL bWarm = FALSE;
    
    if(!hDataAdvise)
       {
       // This is a warm advise 
       int iRetVal;
       
       iRetVal = pMainWnd->MessageBox(
          "Server's Bitmap has changes.\r\nDo you want to update the clients bitmap?",
          "CDDEML Client Sample",MB_YESNO | MB_ICONEXCLAMATION);
       if (iRetVal != IDYES)
           return (HDDEDATA)DDE_FACK;
       
       hDataAdvise = DdeClientTransaction(NULL,0,hszItem, CF_BITMAP, XTYP_REQUEST,
                  1000);
       bWarm = TRUE;                      
       }
    
    // Get bitmap and save into CBitmap
    HBITMAP hBit;
    DdeGetData(hDataAdvise, (LPBYTE)&hBit, sizeof(HBITMAP),0);
    CBitmap *tmpBit = CBitmap::FromHandle(hBit);
    
    // set up DC's
    CDC SrcDC;
    CDC DestDC;
    CDC *WndDC = pMainWnd->GetDC();
    SrcDC.CreateCompatibleDC(WndDC);
    DestDC.CreateCompatibleDC(WndDC);
    pMainWnd->ReleaseDC(WndDC);
    
    // get our bitmap ready 
    BITMAP bm;
    ::GetObject(hBit,sizeof(BITMAP),&bm);
    myBitmap.DeleteObject();
    myBitmap.Detach();
   
    
    // Select bitmaps into DC's
    CBitmap *pOldSrc,*pOldDest;
    
    pOldSrc = SrcDC.SelectObject(tmpBit);
     myBitmap.CreateCompatibleBitmap(&SrcDC,bm.bmWidth,bm.bmHeight);
    pOldDest = DestDC.SelectObject(&myBitmap);
                      
                      
    // Copy Bitmap
    DestDC.BitBlt(0,0,bm.bmWidth, bm.bmHeight ,&SrcDC,0,0,SRCCOPY);
    
    // replace original bitmaps
    SrcDC.SelectObject(pOldSrc);
    DestDC.SelectObject(pOldDest);
    tmpBit->Detach(); 
    
    pMainWnd->Invalidate();
    
    if(bWarm)
       DdeFreeDataHandle( hDataAdvise);
    
    return (HDDEDATA)DDE_FACK;
}     
