//**********************************************************************
//  File name: client.h
//
//  Description: 
//            Declares the CMyClient,  CMainWindow, and CTheApp classes
//            used by the CLIENT application. 
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**********************************************************************

#include "cddecli.h"

//**********************************************************************
//
//  class CMyConv 
//
//  Purpose:
//  
//  Declares the conversation type used by the CLIENT application. 
//  The OnAdvData() function is called by the CDDEClient callback
//  function.  It stores the data in a CBitmap object and invalidates
//  the main window so that it can be repainted.          
//
//  Derived from:  CObject
// 
//  Data Members:  
//    CBitmap myBitmap;          // current bitmap
//    CMainWindow * pMainWnd;    // current window
//
//  Member Functions:
//    HDDEDATA OnAdvData(HSZ hszTopic,HSZ hszItem, 
//                      HDDEDATA hDataAdvise);
//
//  Comments: 
//
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**************************************************************************

class CMainWindow;
class CMyConv : public CDDEClientConv
{
   public:
   CBitmap myBitmap;          // current bitmap
   CMainWindow * pMainWnd;
   HDDEDATA OnAdvData(HSZ hszTopic,HSZ hszItem, HDDEDATA hDataAdvise);

};

//**********************************************************************
//
//  class CMainWindow 
//
//  Purpose:
//  
//  Declares main window class for the CLIENT application.        
//
//  Derived from:  CObject
// 
//  Data Members:  
//     CMyConv myConv;           // DDE conversation  
//     CDDEClientConv sysConv;   // System DDE conversatoin
//     HSZ hszItem;              // item for our conversation       
//
//  Member Functions:
//
//  Constructor
//
//    CMainWindow();
//
//  Menu handlers
//
//    afx_msg void OnPaint();
//    afx_msg void OnAbout();
//    afx_msg void OnWhatDoIDo();
//    afx_msg int OnCreate(LPCREATESTRUCT);
//    afx_msg void OnStartConv();
//    afx_msg void OnStartHot();
//    afx_msg void OnStartWarm();
//    afx_msg void OnStopAdv();
//    afx_msg void OnSysTopics();
//    afx_msg void OnItemList();
//    afx_msg void OnSysItems();
//    afx_msg void OnFormats();
//    afx_msg void OnStatus();
//    afx_msg void OnRtnMsg();
//    afx_msg void OnSvrHelp();
//        
// Helper function
//      void GetSysConvData(LPSTR lpszMessage);
//
//
//  Comments: 
//
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**************************************************************************

class CMainWindow : public CFrameWnd
{
public:
        CMainWindow();

        afx_msg void OnPaint();
        afx_msg void OnAbout();
        afx_msg void OnWhatDoIDo();
        afx_msg int OnCreate(LPCREATESTRUCT);
        afx_msg void OnStartConv();
        afx_msg void OnStartHot();
        afx_msg void OnStartWarm();
        afx_msg void OnStopAdv();
        afx_msg void OnSysTopics();
        afx_msg void OnItemList();
        afx_msg void OnSysItems();
        afx_msg void OnFormats();
        afx_msg void OnStatus();
        afx_msg void OnRtnMsg();
        afx_msg void OnSvrHelp();
        
        // helper function for displaying output for system conversation
        void GetSysConvData(LPSTR lpszMessage);

        CMyConv myConv;           // DDE conversation  
        CDDEClientConv sysConv;   // System DDE conversatoin

        HSZ hszItem;              // item for our conversation
        
        DECLARE_MESSAGE_MAP()
       

};

//**********************************************************************
//
//  class CTheApp 
//
//  Purpose:
//  
//  Declares application class for the CLIENT application.        
//
//  Derived from:  CObject
// 
//  Data Members: 
//     CDDEClient *m_pDDEClient;     // pointer to DDE client 
//
//  Member Functions:
// 
//     BOOL InitInstance();
//     int  ExitInstance();
//
//  Comments: 
//
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**************************************************************************

class CTheApp : public CWinApp
{
public:
        CDDEClient *m_pDDEClient;
        
        BOOL InitInstance();
        int  ExitInstance();
};

