// COPYRIGHT:
//
//   (C) Copyright Microsoft Corp. 1993.  All rights reserved.
//
//   You have a royalty-free right to use, modify, reproduce and
//   distribute the Sample Files (and/or any modified version) in
//   any way you find useful, provided that you agree that
//   Microsoft has no warranty obligations or liability for any
//   Sample Application Files which are modified.
//
#include <afxwin.h>
#include <afxcoll.h>
#include "resource.h"
#include "dynatest.h"

#include "inputdlg.h"

//
// CMainWindow message map:
//
BEGIN_MESSAGE_MAP( CMainWindow, CFrameWnd )	
	ON_COMMAND( IDM_EXIT, OnExit )
	ON_COMMAND( IDM_ABOUT, OnAbout )
   ON_COMMAND( IDM_MSGBOX, OnMsgBox ) 
	ON_COMMAND( IDM_BEEP, OnBeep ) 
	ON_COMMAND( IDM_CHECK, OnCheck )
	
END_MESSAGE_MAP()

/////////////////////////////////////
//
// Dynamic Menu Functions
//
void MessageFunc(UINT nID, CWnd*) ;
void BeepFunc(UINT nID, CWnd*) ;
void CheckFunc(UINT nID, CWnd* pwnd) ;

void MessageFunc(UINT nID, CWnd*) 
{
	MessageBox(NULL, "This is a dynamic menu.","DynaMenu", MB_OK) ;
}

void BeepFunc(UINT nID, CWnd*) 
{
	MessageBeep(0) ;
}

void CheckFunc(UINT nID, CWnd* pwnd)
{
	CMenu* pMainMenu = pwnd->GetMenu() ;
	UINT state = pMainMenu->GetMenuState(nID, MF_BYCOMMAND) ;
	ASSERT( state != -1) ;
	
	pMainMenu->CheckMenuItem(nID,
			(state && MF_CHECKED) ? MF_UNCHECKED : MF_CHECKED ) ;
		
}

/////////////////////////////////////
//
// Menu Handlers
//
void CMainWindow::OnMsgBox()
{
	CString aString ;
	CInputDlg aDlg(this, aString) ;
	
	if( aDlg.DoModal() == IDOK )
		aDynaMenu.AddMenu(MessageFunc, (const char*)aString ) ;
}

void CMainWindow::OnBeep()
{
	CString aString ;
	CInputDlg aDlg(this, aString) ;
	
	if( aDlg.DoModal() == IDOK )
		aDynaMenu.AddMenu(BeepFunc, (const char*)aString ) ;

}

void CMainWindow::OnCheck()
{
	CString aString ;
	CInputDlg aDlg(this, aString) ;
	
	if( aDlg.DoModal() == IDOK )
		aDynaMenu.AddMenu(CheckFunc, (const char*)aString ) ;
}

//////////////////////////////////
// theApp:
// Just creating this application object runs the whole application.
//
CTheApp theApp;

//////////////////////////////////
//
// CMainWindow constructor:
//
CMainWindow::CMainWindow()
{
	LoadAccelTable( "MainAccelTable" );
	Create( NULL, "Dynamic Menu Sample Application",
		WS_OVERLAPPEDWINDOW, rectDefault, NULL, "MainMenu" );
}

//
// OnAbout:
//
void CMainWindow::OnAbout()
{
	CModalDialog about( "AboutBox", this );
	about.DoModal();
}

//
// OnExit
//
void CMainWindow::OnExit()
{
	DestroyWindow() ;
}

/////////////////////////////////////////////////////////////////////////////
// CTheApp
//
//
BOOL CTheApp::InitInstance()
{
	m_pMainWnd = new CMainWindow();
	m_pMainWnd->ShowWindow( m_nCmdShow );
	m_pMainWnd->UpdateWindow();

	return TRUE;
}

CTheApp::~CTheApp()
{
	
}
