/*
    global.h

    Main application header file

*/

#include <windows.h>
#include <shellapi.h>
#include <commdlg.h>
#include <stdlib.h>
#include <memory.h>
#include "dialogs.h"

//
// Menu ids
//

#define IDM_OPEN            0x0100
#define IDM_SAVE            0x0101
#define IDM_SAVEAS          0x0102
#define IDM_PRINT           0x0103
#define IDM_PRINTSETUP      0x0104
#define IDM_EXIT            0x0105
#define IDM_NEW             0x0106

#define IDM_CUT             0x0200
#define IDM_COPY            0x0201
#define IDM_PASTE           0x0203

#define IDM_DEBUG0          0x0800
#define IDM_DEBUG1          (IDM_DEBUG0+1)
#define IDM_DEBUG2          (IDM_DEBUG0+2)
#define IDM_DEBUG3          (IDM_DEBUG0+3)
#define IDM_DEBUG4          (IDM_DEBUG0+4)
#define IDM_DEBUG_DIRTY     (IDM_DEBUG0+5)

#define IDM_HELPCONTENTS    0x0900
#define IDM_ABOUT           0x0901

//
// general constants
//



//
// Child window ids
//


//
// global data
//

char *szAppName;                    // App name
HINSTANCE hAppInstance;             // app instance
HWND hwndMain;                      // main window handle
TEXTMETRIC tmSysFont;               // system font metrics
HANDLE hAccTable;                   // accelerator table

//
// main.c
//

extern LRESULT CALLBACK MainWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
extern UINT cdecl Message(UINT uiBtns, LPSTR lpFormat, ...) ;

//
// init.c
//

extern BOOL InitFirstInstance(HANDLE hInstance);
extern BOOL InitCurrentInstance(HANDLE hInstance, LPSTR lpszCmdLine, int cmdShow);
extern void Terminate(void);

//
// help.c
//

extern void About(HWND hWnd);
extern void Help(HWND hWnd, WORD wParam);

//
// file.c
//

// flags for File functions
#define FILE_PROMPT_FOR_SAVE        0x0001
#define FILE_DONT_PROMPT_FOR_SAVE   0x0000
#define FILE_UPDATE_CAPTION         0x0002
#define FILE_DONT_UPDATE_CAPTION    0x0000
#define FILE_SAVE                   0x0000
#define FILE_SAVE_AS                0x0004

// global file data
BOOL gfDirty;
char gachFileName[_MAX_PATH];
BOOL gfUntitled;
LPSTR pFileData;
DWORD dwFileLength;

// functions
extern BOOL FileNew(WORD wFlags);
extern BOOL FileOpen(LPSTR szFileName);
extern BOOL FileSave(WORD wFlags);

//
// clip.c
//

extern void ClipCommand(HWND hWnd, WPARAM aParam);
extern void InitClipMenu(HMENU hmenuPopup);

//
// print.c
//

extern void Print(HDC hDC);

//
// Debug support
//

extern LRESULT CALLBACK DebugWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

#ifdef DEBUG

    DWORD   __dwEval;
    int __iDebugLevel;

    extern void SetDebugLevel(int i);
    extern void __AssertMsg(LPSTR exp ,LPSTR file, int line);
    extern void cdecl DbgOut(LPSTR lpFormat, ...);

    #define ASSERT(exp) \
        ((exp) ? (void)0 : __AssertMsg((LPSTR)(#exp), (LPSTR)__FILE__, __LINE__))
    #define EVAL(exp) \
        (__dwEval=(DWORD)(exp), (__dwEval) ? (void)0 : __AssertMsg((LPSTR)(#exp), (LPSTR)__FILE__, __LINE__), __dwEval)

    #define dprintf                          DbgOut
    #define dprintf1 if (__iDebugLevel >= 1) DbgOut
    #define dprintf2 if (__iDebugLevel >= 2) DbgOut
    #define dprintf3 if (__iDebugLevel >= 3) DbgOut
    #define dprintf4 if (__iDebugLevel >= 4) DbgOut

#else // not DEBUG

    #define ASSERT(exp) 0
    #define EVAL(exp) (exp)

    #define dprintf  if (0) ((int (*)(char *, ...)) 0)
    #define dprintf1 if (0) ((int (*)(char *, ...)) 0)
    #define dprintf2 if (0) ((int (*)(char *, ...)) 0)
    #define dprintf3 if (0) ((int (*)(char *, ...)) 0)
    #define dprintf4 if (0) ((int (*)(char *, ...)) 0)

#endif // DEBUG
