/*
    init.c

    The initialization and termination routines

*/

#include "global.h"

//
// Initialise the first instance
//

BOOL InitFirstInstance(HANDLE hInstance)
{
    WNDCLASS wc;

    
    //
    // define the class of window we want to register
    //

    wc.lpszClassName    = szAppName;
    wc.style            = CS_HREDRAW | CS_VREDRAW;
    wc.hCursor          = LoadCursor(NULL, IDC_ARROW);
    wc.hIcon            = LoadIcon(hInstance,MAKEINTRESOURCE(IDI_APP));
    wc.lpszMenuName     = "Menu";
    wc.hbrBackground    = GetStockObject(WHITE_BRUSH);
    wc.hInstance        = hInstance;
    wc.lpfnWndProc      = MainWndProc;
    wc.cbClsExtra       = 0;
    wc.cbWndExtra       = 0;
    
    if (!RegisterClass(&wc)) {
        return FALSE;
    }

#ifdef DEBUG

    wc.lpszClassName    = "Debug";
    wc.style            = CS_HREDRAW | CS_VREDRAW;
    wc.hCursor          = LoadCursor(NULL, IDC_ARROW);
    wc.hIcon            = LoadIcon(hInstance,"Icon");
    wc.lpszMenuName     = NULL;
    wc.hbrBackground    = GetStockObject(WHITE_BRUSH);
    wc.hInstance        = hInstance;
    wc.lpfnWndProc      = DebugWndProc;
    wc.cbClsExtra       = 0;
    wc.cbWndExtra       = 0;
    
    return RegisterClass(&wc);

#endif // DEBUG

}

//
// Init the current instance
//

BOOL InitCurrentInstance(HANDLE hInstance, LPSTR lpszCmdLine, int cmdShow)
{
    HDC hDC;

    //
    // Get any system metrics we use a lot
    //

    hDC = GetDC(NULL);  // get a DC
    GetTextMetrics(hDC, &tmSysFont);
    ReleaseDC(NULL, hDC);

    //
    // Create any GDI objects we use a lot
    //





    //
    // Create the main window
    //

    hwndMain = CreateWindow(szAppName,
                            szAppName,
                            WS_OVERLAPPEDWINDOW,
                            CW_USEDEFAULT,
                            0,
                            GetSystemMetrics(SM_CXSCREEN) / 2,
                            GetSystemMetrics(SM_CYSCREEN) / 4,
                            (HWND)NULL,
                            (HMENU)NULL,
                            hInstance,
                            (LPSTR)NULL
                            );
    
    if (!hwndMain) {
        return FALSE;
    }

    ShowWindow(hwndMain, cmdShow);
    UpdateWindow(hwndMain);

    //
    // Load the accelerator table
    //

    hAccTable = LoadAccelerators(hInstance, "AccTable");
    if (!hAccTable) {
        return FALSE;
    }

    //
    // Create a new document
    //

    FileNew(FILE_DONT_PROMPT_FOR_SAVE | FILE_UPDATE_CAPTION);

    //
    // Open any file given in the command line
    //

    if (lpszCmdLine && lstrlen(lpszCmdLine)) {
        FileOpen(lpszCmdLine);
    }

    //
    // Say we do drag and drop
    //

    DragAcceptFiles(hwndMain, TRUE);

#ifdef DEBUG

    //
    // Set up the debug menu state
    //

    SetDebugLevel(__iDebugLevel);   // set menu state

#endif

    dprintf2("started");

    return TRUE;
}

//
// Close down and tidy up
//

void Terminate()
{
    char buf[256];

    //
    // Say we don't drag and drop any more
    //

    DragAcceptFiles(hwndMain, FALSE);

    //
    // Delete any GDI objects
    //






#ifdef DEBUG

    //
    // Save the debug level
    //

    wsprintf(buf, "%d", __iDebugLevel);
    WriteProfileString(szAppName, "debug", buf);
    dprintf2("ended");

#endif // DEBUG
        
}
