//*************************************************************
//  File name: mfcdib.h
//
//  Description:
//     Declares the interface for the sample application
//
//  History:    Date       Author     Comment
//              05/15/92   Hung H. Nguyen      Created
//                         Brian Scott
//
// Written by Microsoft Product Support Services, Windows Developer Support
// Copyright (c) 1992 Microsoft Corporation. All rights reserved.
//*************************************************************
#ifndef __MFCDIB_H__
#define __MFCDIB_H__

#include "cdib.h"

//*************************************************************
//  Class:
//      CMainWindow
//
//  Description:
//      The main window class.
//
//  Derived from:
//      CFrameWnd
//
//  Data Members:
//      int cxScreen      : Holds the maximum width of the screen
//      int cyScreen      : Holds the maximum height of the screen
//
//
//  Member Functions:
//
//      SizeWindow()      : Resizes the window to the size of the DIB
//
//      CMainWindow()     : Constructor
//      OnPaint()         : Handles DIB painting
//      OnOpen()          : Lets user open and load a .BMP file
//      OnSave()          : Lets user save a .BMP file
//      OnExit()          : Exits the app
//      OnCopyDIB()       : Copies the DIB to the clipboard as a DIB
//      OnPasteDIB()      : Pastes a DIB from the clipboard (CF_DIB)
//      OnCopyBM()        : Copies the DIB to the clipboard as a bitmap
//      OnPasteBM()       : Pastes a bitmap from the clipboard
//      OnRead()          : Reads serialized CDIB object from disk
//      OnWrite()         : Writes CDIB object to disk
//      OnPaletteChanged(): Handles WM_PALETTECHANGED message
//      OnQueryNewPalette(): Handles WM_QUERYNEWPALETTE message
//      OnInitMenu()      : Enables/disables copy/paste to/from clipboard
//      OnAbout()         : Message handler for WM_COMMAND,IDM_ABOUT
//      OnInstructions()  : Message handler for WM_COMMAND,IDM_INSTRUCTIONS
//
//  Comments
//
//  History:    Date       Author     Comment
//              05/15/92   Hung H. Nguyen      Created
//                         Brian Scott
//
//*************************************************************  
class CMainWindow : public CFrameWnd
{
private:
    int         cxScreen;
    int         cyScreen;

    void SizeWindow(void);

public:
    CMainWindow();

    BOOL CopyToClipboard(BOOL bDIB = TRUE);
    BOOL PasteFromClipboard(BOOL bDIB = TRUE);

    afx_msg void OnPaint();
    afx_msg void OnOpen();
    afx_msg void OnSave();
    afx_msg void OnExit();
    afx_msg void OnCopyDIB();
    afx_msg void OnPasteDIB();
    afx_msg void OnCopyBM();
    afx_msg void OnPasteBM();
    afx_msg void OnRead();
    afx_msg void OnWrite();
    afx_msg void OnPaletteChanged(CWnd*);
    afx_msg BOOL OnQueryNewPalette();
    afx_msg void OnInitMenu(CMenu*);
    afx_msg void OnAbout();
    afx_msg void OnInstructions();

    DECLARE_MESSAGE_MAP()
};


//*************************************************************
//  Class:
//      CApp
//
//  Description:
//      The Application class
//
//  Derived from:
//      CWinApp
//
//  Data Members:
//      m_cDIB            : Holds the CDIB object
//
//  Member Functions:
//      InitInstance()    : Creates the main window and calls necessary
//                        : show the window 
//
//  Comments
//
//  History:    Date       Author     Comment
//              05/15/92   Hung H. Nguyen      Created
//                         Brian Scott
//
//*************************************************************  
class CApp : public CWinApp
{
public:
    CDIB        m_cDIB;

    BOOL InitInstance();
};

//*************************************************************
//  Class:
//      CInstructionBox
//
//  Description:
//      The dialog box used to display the instructions to the sample
//
//  Derived from:
//      CModalDialog
//
//  Data Members:
//      m_Text         : The instruction text
//      m_TitleFont    : The font used for the title
//
//  Member Functions:
//      CInstructionBox: Constructor, Constructs base class as well
//      OnInitDialog   : Used to set the text of the dialog box
//                     : and the font used for the title
//  Comments
//
//  History:    Date       Author     Comment
//              05/15/92   Hung H. Nguyen      Created
//                         Brian Scott
//
//*************************************************************  
class CInstructionBox : public CModalDialog
{
private:
    CString m_Text;
    CFont   m_TitleFont;

public:
    CInstructionBox( const char* name, CWnd* Parent )
                  : CModalDialog( name,Parent ){}
    virtual BOOL OnInitDialog();
};



#endif // __MFCDIB_H__
