// COPYRIGHT:
//
//   (C) Copyright Microsoft Corp. 1993.  All rights reserved.
//
//   You have a royalty-free right to use, modify, reproduce and
//   distribute the Sample Files (and/or any modified version) in
//   any way you find useful, provided that you agree that
//   Microsoft has no warranty obligations or liability for any
//   Sample Application Files which are modified.
//
#include <afxwin.h>
#include "owin.h"
#include "owinres.h"
#include "inputdlg.h"


/////////////////////////////////////////////
// Message Map
//
BEGIN_MESSAGE_MAP( COutputWindow, CFrameWnd )

// Windows Messages
	ON_WM_PAINT()
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_HSCROLL()
	ON_WM_VSCROLL()

// Menu Handlers
	ON_COMMAND( IDM_FONT,OnFont )
	ON_COMMAND( IDM_CLEAR, OnClearBuffer )
	
END_MESSAGE_MAP()

//////////////////////////////////////////
// Constructor
//
//
COutputWindow::COutputWindow()
{
	maxLines = DefaultMaxLines ;
	maxColumns = DefaultMaxColumns ;

	// Put a blank line in the list 
		lineList.AddTail("") ;		
}

//////////////////////////////////////////
// Destructor
//
//
COutputWindow::~COutputWindow()
{
	TRACE("Calling Destructor\r\n") ;
}

/////////////////////////////////////////////////////////////
// Create -- 
//
//
BOOL COutputWindow::Create(const char FAR* lpWindowName )
{
	if ( !CFrameWnd::Create( NULL, lpWindowName, 
						 WS_VISIBLE | WS_OVERLAPPEDWINDOW  
						| WS_VSCROLL | WS_HSCROLL,
						rectDefault, NULL, "owinMenu") )
	{
			TRACE(" COutputWindow Failed create(). \r\n") ;
			return FALSE ;
	}
	
	// Disable Close Menu Option
		CMenu* pMenu = GetSystemMenu(FALSE) ;
		pMenu->EnableMenuItem( SC_CLOSE, MF_GRAYED ) ;
		
	// Display the Window
		ShowWindow( SW_SHOWNORMAL ) ;
		UpdateWindow() ;
		
	return TRUE ;
}

/////////////////////////////////////////////////////////////
// GetInput -- 
//
//
int COutputWindow::GetInput(char *someChars, int count) 
{
	if (inputBuffer.IsEmpty())
	{
		OnVScroll(SB_BOTTOM,0,NULL) ;
	
		BOOL WasIcon = IsIconic() ;
		if (WasIcon) OpenIcon() ;
	
		// Run Input Dialog
			CInputDlg adlg(this, inputBuffer) ;
			if( adlg.DoModal() != IDOK)
			{
				if (WasIcon) CloseWindow() ;
				return 0 ;
			}

		Display(inputBuffer, inputBuffer.GetLength() ) ;
		if (WasIcon) CloseWindow() ;
	}
		
		// copy characters to someChars
		int inputBufferLength = inputBuffer.GetLength() ;
		int numChars = min(count, inputBufferLength) ;
		int remainingBuffer = inputBufferLength - numChars ;
		
		char * ptr = inputBuffer.GetBuffer(inputBufferLength) ;
			memmove(someChars,ptr, numChars) ;
			if (remainingBuffer > 0)
				memmove(ptr, ptr+numChars, remainingBuffer ) ;
		inputBuffer.ReleaseBuffer(remainingBuffer) ;
		
		if (remainingBuffer <= 0 )
			inputBuffer.Empty() ;
				
		return numChars ;
	   
}
/////////////////////////////////////////////////////////////
// ReturnInput -- 
//
//
BOOL COutputWindow::ReturnInput( char *someChars, int count )
{
	// Expensive
	// Put in exception handler
		
	CString insertString(someChars, count) ;
	inputBuffer = insertString + inputBuffer ;
	return TRUE ;
}

/////////////////////////////////////////////////////////////
// FlushInput -- 
//
//
BOOL COutputWindow::FlushInput()
{
	inputBuffer.Empty() ;
	return TRUE ;
}
///////////////////////////////////////////////////////////////
// Display -- 
//
//
void COutputWindow::Display(const char *someChars, int count)
{
	ASSERT( someChars != NULL ) ;
	
	BOOL IsLineFeed = FALSE ;

	int column = lineList.GetTail().GetLength() ;	
	char *pDest = lineList.GetTail().GetBufferSetLength(maxColumns) + column;
	
	for( const char *pSrc = someChars ; pSrc < someChars + count ; pSrc++ )
	{
		switch(*pSrc)
		{
			case '\n': // lf
				IsLineFeed = TRUE ;
				break ;			
			case '\r': // cr
				break;			
			case '\t': // tab
				*pDest++ = ' ' ;
				column++ ;
				while( (column%TABNUM) && (column < maxColumns) )
				{
					*pDest++ = ' ' ;
					column++ ;
				}
				break;			
			default:
				*pDest++ = *pSrc ;
				column++ ;
				break ;
		} //switch
			
		if ( (column >= maxColumns) || (IsLineFeed) )
		{
			IsLineFeed = FALSE ;
			
			lineList.GetTail().ReleaseBuffer( column ) ;
			
			lineFeed() ;

			column = lineList.GetTail().GetLength() ;
			ASSERT(column == 0) ;
			
			pDest = lineList.GetTail().GetBufferSetLength(maxColumns);						
			
		}

	} //for
	
	lineList.GetTail().ReleaseBuffer(column) ;
	
	OnSize(0, cxClient, cyClient) ;
}

/////////////////////////////////////////////
// scrollOffTop
//
void COutputWindow::scrollOffTop() 
{
		if( lineList.GetCount() >= maxLines ) 
		{
			// Scroll Lines off the top
				lineList.RemoveHead() ;
		}
}

/////////////////////////////////////////////
// lineFeed
//
void COutputWindow::lineFeed()
{
	lineList.AddTail("") ;
	scrollOffTop() ;
}

//
// calcTextMetrics
//
void COutputWindow::calcTextMetrics()
{
	TEXTMETRIC tm ;

	// Setup Font
		CFont aFont ;
		aFont.CreateFontIndirect(&lf) ;
	
	// Get the text emtrics.
		CDC* dc = GetDC() ;
			CFont* pOldFont = dc->SelectObject(&aFont) ;
			dc->GetTextMetrics( &tm ) ;
			dc->SelectObject(pOldFont) ;
		ReleaseDC( dc ) ;
		
	// Calculate Displayables
		cxChar = tm.tmAveCharWidth ;
		cxCaps = ( (tm.tmPitchAndFamily & 1 ) ? 3 : 2 ) * cxChar / 2 ;
		cyChar = tm.tmHeight + tm.tmExternalLeading ;
		nMaxWidth = (maxColumns + 1) * cxCaps ;
		nVscrollPos = 0 ;
		nHscrollPos = 0 ;
}

////////////////////////////Menu Handlers///////////////////////////
//
//

//
// OnClearBuffer
//
void COutputWindow::OnClearBuffer()
{
	inputBuffer.Empty() ; //??
	
	lineList.RemoveAll();
	lineFeed() ;
	
	OnSize(0, cxClient, cyClient) ;
}

//
// OnFont
//
void COutputWindow::OnFont()
{
	CFontDialog fontDialog(&lf, CF_EFFECTS | CF_SCREENFONTS | CF_ANSIONLY
					| CF_FORCEFONTEXIST, NULL, this ) ;
		
   if(fontDialog.DoModal() == IDOK )
	{
		lf = fontDialog.m_lf ;
		rgbTextColor = fontDialog.GetColor() ;
		
		calcTextMetrics() ;
		OnSize(0, cxClient, cyClient) ;
	}
}



////////////////////////////Message Handlers///////////////////////////
//
//
//

//
// OnCreate
//
int COutputWindow::OnCreate( LPCREATESTRUCT cs )
{
	// Store current text color
		rgbTextColor = GetSysColor( COLOR_WINDOWTEXT ) ;
	
	// Start with the system Font ;
		HFONT hFont = GetStockObject(ANSI_FIXED_FONT) ;
		GetObject(hFont, sizeof(LOGFONT), &lf) ;		
		
	// Calculate text sizes
		calcTextMetrics() ;
	
	return 0 ;
}

//
// OnPaint
//
void COutputWindow::OnPaint()
{
	CPaintDC dc( this ) ;
	
	// Do font stuff
		CFont aFont ;
		aFont.CreateFontIndirect(&lf) ;
		CFont* pOldFont = dc.SelectObject(&aFont) ; 

	// Set text and background colors
		dc.SetTextColor( rgbTextColor ) ;
		dc.SetBkColor( GetSysColor( COLOR_WINDOW ) ) ;

		dc.SetTextAlign( TA_LEFT | TA_TOP ) ;

		
	// Compute the lines that need to be redrawn
		int nStart = max( 0, nVscrollPos + dc.m_ps.rcPaint.top /
						cyChar - 1) ;
					
		int nEnd = min( (int) lineList.GetCount(),
			nVscrollPos + ( dc.m_ps.rcPaint.bottom / cyChar + 1 ) ) ;
		
	int x = cxChar * (-nHscrollPos ) ;
	int y = cyChar * (nStart - nVscrollPos ) ;
	
	CString aLine ;
	for(int nCount = nStart ; nCount < nEnd ; nCount++)
	{	
		// Get Line
			aLine = lineList.GetAt(lineList.FindIndex( nCount ) );
		
		// display line
			dc.TextOut( x, y, (const char *)aLine, aLine.GetLength() ) ;
		
		// Get ready for next line	
			y += cyChar ;
	}
	
	// Why doesn't MFC clean up for me?
		dc.SelectObject(pOldFont) ;
}

//
// OnSize
//
void COutputWindow::OnSize( UINT type, int x, int y)
{
	cxClient = x ;
	cyClient = y ;
	
	nVscrollMax = max( 0, (int)(lineList.GetCount() ) - cyClient / cyChar ) ;
	nHscrollMax = max( 0, ( nMaxWidth - cxClient ) / cxChar ) ;
	
	// Recaluate Vertical scroll position.
		nVscrollPos = min( nVscrollPos, nVscrollMax ) ;

		SetScrollRange( SB_VERT, 0, nVscrollMax, FALSE ) ;
		SetScrollPos( SB_VERT, nVscrollPos, TRUE ) ;

	// Recaluate Horizontal scroll position
		nHscrollPos = min( nHscrollPos, nHscrollMax ) ;

		SetScrollRange( SB_HORZ, 0, nHscrollMax, FALSE ) ;
		SetScrollPos( SB_HORZ, nHscrollPos, TRUE ) ;
		
	Invalidate(TRUE) ;
}

//
// OnHScroll
//
void COutputWindow::OnHScroll( UINT how, UINT pos, CScrollBar* control ) 
{
	short nScrollInc ;
	
	switch( how )
	{
		case SB_LINEUP:
			nScrollInc = -1;
			break;

		case SB_LINEDOWN:
			nScrollInc = 1;
			break;

		case SB_PAGEUP:
			nScrollInc = - ((cxClient / cxChar) / 8 ) ;
			break;

		case SB_PAGEDOWN:
			nScrollInc = (cxClient / cxChar) / 8 ;
			break;

		case SB_THUMBPOSITION:
			nScrollInc = pos - nHscrollPos;
			break;

		default:
			nScrollInc = 0;
	}

	if ( nScrollInc = max( -nHscrollPos,
			 min( nScrollInc, nHscrollMax - nHscrollPos ) ) )
	{
		nHscrollPos += nScrollInc;
		ScrollWindow( -cxChar * nScrollInc, 0 );
		SetScrollPos( SB_HORZ, nHscrollPos );
		UpdateWindow();
	}

}

//
// OnVScroll
//
void COutputWindow::OnVScroll( UINT how, UINT pos, CScrollBar* control )
{
	short nScrollInc;
	
	switch ( how )
	{
		case SB_TOP:
			nScrollInc = -nVscrollPos;
			break;

		case SB_BOTTOM:
			nScrollInc = nVscrollMax - nVscrollPos;
			break;
			
		case SB_LINEUP:
			nScrollInc = -1;
			break;

		case SB_LINEDOWN:
			nScrollInc = 1;
			break;

		case SB_PAGEUP:
			nScrollInc = min( -1, -cyClient / cyChar );
			break;

		case SB_PAGEDOWN:
			nScrollInc = max( 1, cyClient / cyChar );
			break;

		case SB_THUMBTRACK:
			nScrollInc = pos - nVscrollPos;
			break;

		default:
			nScrollInc = 0;
	}

	if ( nScrollInc = max( -nVscrollPos,
			min( nScrollInc, nVscrollMax - nVscrollPos ) ) )
	{
		nVscrollPos += nScrollInc;
		ScrollWindow( 0, -cyChar * nScrollInc );
		SetScrollPos( SB_VERT, nVscrollPos );
		UpdateWindow();
	}
}


