//**********************************************************************
//  File name: Server.cpp
//
//  Description: 
//            Declares member functions and static data for the 
//            CServerClient, CMyConv, CSystemConv, CMainWindow, 
//            and CTheApp classes used by the SERVER application.
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**********************************************************************

// COPYRIGHT:
//
//   (C) Copyright Microsoft Corp. 1993.  All rights reserved.
//
//   You have a royalty-free right to use, modify, reproduce and
//   distribute the Sample Files (and/or any modified version) in
//   any way you find useful, provided that you agree that
//   Microsoft has no warranty obligations or liability for any
//   Sample Application Files which are modified.
//

#include <afxwin.h>
#include "resource.h"

#include "server.h"

// Just creating this application object runs the whole application.
CTheApp theApp;

//***********************************************************
//
//  CMyServer :: CMyServer()
//
//  Purpose:  Constructor - calls Initialize
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
CMyServer::CMyServer()
{
Initialize();
}

//***********************************************************
//
//  CMyServer :: ~CMyServer()
//
//  Purpose:  Destructor - calls Uninitialize
//
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
CMyServer::~CMyServer()
{
Uninitialize();
}

//***********************************************************
//
//  CMyServer :: Initialize()
//
//  Purpose:  Creates topic tables and initializes server
//
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMyServer::Initialize()
{
   CDDEServer::Initialize();
   hszTopics[SYSTEM_TOPIC] = DdeCreateStringHandle( (LPSTR)SZDDESYS_TOPIC);
   hszTopics[BITMAP_TOPIC] = DdeCreateStringHandle( "Sample");
   hszServ = DdeCreateStringHandle( "DDEMLDemo" );
   DdeNameService(hszServ,NULL,DNS_REGISTER);  
   CSystemConv::Init();
}

//***********************************************************
//
//  CMyServer :: Uninitialize()
//
//  Purpose: Cleans up topic tables and unitializes server
//
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMyServer::Uninitialize()
{
   CSystemConv::CleanUp();
   DdeNameService(hszServ,NULL,DNS_UNREGISTER);
   DdeFreeStringHandle(hszServ);  
   DdeFreeStringHandle(hszTopics[SYSTEM_TOPIC]);
   DdeFreeStringHandle(hszTopics[BITMAP_TOPIC]);
   
   CDDEServer::Uninitialize();
}

//***********************************************************
//
//  CMyServer :: OnConnect()
//
//  Purpose: Handles clients attempts to connect.
//
//  Parameters: HSZ hszTopic
//              HSZ hszService
//              CONVCONTEXT FAR *pcc
//              BOOL fSameInstance   
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
BOOL CMyServer::OnConnect(HSZ hszTopic,HSZ hszService,CONVCONTEXT FAR *pcc,BOOL fSameInstance)
{
   
   for(int i=0;i<TOPICS;++i)
      {
      if(!DdeCmpStringHandles(hszTopic,hszTopics[i]))
         return TRUE;
      }
   return FALSE;
}   


//***********************************************************
//
//  CMyServer :: OnWildConnect()
//
//  Purpose: Handles clients attempts to connect.
//
//  Parameters: UINT wFmt
//              HSZ hszTopic
//              HSZ hszService
//              CONVCONTEXT FAR *pcc
//              BOOL fSameInstance  
//
//  Return: (HDDEDATA)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
HDDEDATA CMyServer::OnWildConnect(UINT wFmt,HSZ hszTopic,HSZ hszService,CONVCONTEXT FAR *pcc,BOOL fSameInstance)
{
   
   if((hszService != hszServ) && (hszService != NULL))
      return (HDDEDATA)NULL;
      
   HSZPAIR ahszp[TOPICS + 1];
   int i;
   int j=0;
   
   for(i=0;i<TOPICS;++i)
   {
      if(!DdeCmpStringHandles(hszTopic,hszTopics[i]))
         {
         ahszp[j].hszSvc = hszServ;
         ahszp[j++].hszTopic = hszTopics[i];
         }
   }
   
   ahszp[j].hszSvc =  ahszp[j].hszTopic  = NULL; 
   ++j;
   
   return(DdeCreateDataHandle((LPBYTE)ahszp,sizeof(HSZPAIR) * j,0L,0,wFmt,0));
}


//***********************************************************
//
//  CMyServer :: OnConnectConfirm()
//
//  Purpose: Creates conversations.
//
//  Parameters: HCONV hConv
//              HSZ hszTopic
//              HSZ hszService
//              BOOL fSameInstance   
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMyServer::OnConnectConfirm(HCONV hConv,HSZ hszTopic,HSZ hszService,BOOL fSameInstance)
{
     CDDEServerConv *pConv;
   
     if(!DdeCmpStringHandles(hszTopic,hszTopics[SYSTEM_TOPIC]))
         pConv = new CSystemConv;
     if(!DdeCmpStringHandles(hszTopic,hszTopics[BITMAP_TOPIC]))
         pConv = new CMyConv;  
         
     pConv->Create(hConv, hszTopic, hszService, fSameInstance );
}


// static item and data tables
HSZ CSystemConv::hszItems[N_ITEMS];
HDDEDATA CSystemConv::hData[N_ITEMS];

//***********************************************************
//
//  CSystemConv :: Init()
//
//  Purpose: Initializes item and data tables for system 
//            conversation.
//
//  Parameters: 
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CSystemConv::Init()
{
   char buf[256];
   
   hszItems[0] = CMyServer::m_pServer->DdeCreateStringHandle( SZDDESYS_ITEM_TOPICS);
   lstrcpy(buf, "Sample, System" );          
   hData[0] = CMyServer::m_pServer->DdeCreateDataHandle(buf,lstrlen(buf)+1,0,hszItems[0],
                   CF_TEXT,HDATA_APPOWNED);

   hszItems[1] = CMyServer::m_pServer->DdeCreateStringHandle( SZDDESYS_ITEM_SYSITEMS);
   lstrcpy(buf, "Formats, Help, ItemList, RtnMsg, Status, SysItems, Topics" );
   hData[1] = CMyServer::m_pServer->DdeCreateDataHandle(buf,lstrlen(buf)+1,0,hszItems[1],
                   CF_TEXT,HDATA_APPOWNED);


   hszItems[2] = CMyServer::m_pServer->DdeCreateStringHandle( SZDDESYS_ITEM_RTNMSG);
   lstrcpy(buf, "Not Available");
   hData[2] = CMyServer::m_pServer->DdeCreateDataHandle(buf,lstrlen(buf)+1,0,hszItems[2],
                   CF_TEXT,HDATA_APPOWNED);

  
   hszItems[3] = CMyServer::m_pServer->DdeCreateStringHandle( SZDDESYS_ITEM_STATUS);
   lstrcpy(buf,   "Server is READY" );
   hData[3] = CMyServer::m_pServer->DdeCreateDataHandle(buf,lstrlen(buf)+1,0,hszItems[3],
                   CF_TEXT,HDATA_APPOWNED);

  
   hszItems[4] = CMyServer::m_pServer->DdeCreateStringHandle( SZDDESYS_ITEM_FORMATS);
   lstrcpy(buf, "CF_TEXT, CF_BITMAP" );
   hData[4] = CMyServer::m_pServer->DdeCreateDataHandle(buf,lstrlen(buf)+1,0,hszItems[4],
                   CF_TEXT,HDATA_APPOWNED);

   hszItems[5] = CMyServer::m_pServer->DdeCreateStringHandle(SZDDESYS_ITEM_HELP);
   lstrcpy(buf,  "Not Implemented" );
   hData[5] = CMyServer::m_pServer->DdeCreateDataHandle(buf,lstrlen(buf)+1,0,hszItems[5],
                   CF_TEXT,HDATA_APPOWNED);

   hszItems[6] = CMyServer::m_pServer->DdeCreateStringHandle(SZDDE_ITEM_ITEMLIST);
   lstrcpy(buf, "NumWindowsDisplayed, ChangeBitmap" );
   hData[6] = CMyServer::m_pServer->DdeCreateDataHandle(buf,lstrlen(buf)+1,0,hszItems[6],
                   CF_TEXT,HDATA_APPOWNED); 
}    

//***********************************************************
//
//  CSystemConv :: CleanUp()
//
//  Purpose: Cleans up item and data tables for system 
//            conversation.
//
//  Parameters: 
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CSystemConv::CleanUp() 
{
   for(int i=0;i<N_ITEMS;++i)
   {
      CMyServer::m_pServer->DdeFreeDataHandle(hData[i]);
      CMyServer::m_pServer->DdeFreeStringHandle(hszItems[i]);
   }
}

//***********************************************************
//
//  CSystemConv :: OnRequest()
//
//  Purpose: Passes system data to client
//
//  Parameters:  UINT wFmt
//               HSZ hszTopic
//               HSZ hszItem
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
HDDEDATA CSystemConv::OnRequest( UINT wFmt, HSZ hszTopic,HSZ hszItem)
{
   for(int i = 0;i<N_ITEMS;++i)
      if(!DdeCmpStringHandles(hszItem,hszItems[i]))
         return hData[i];
   
   return NULL;
}


// static item and data   
HDDEDATA CMyConv::hBitmapData[3];
HSZ CMyConv::hszItem;
CMainWindow * CMyConv::m_pMainWnd;

//***********************************************************
//
//  CMyConv :: OnRequest()
//
//  Purpose: Passes data to client
//
//  Parameters:  UINT wFmt
//               HSZ hszTopic
//               HSZ hszItem
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
HDDEDATA CMyConv::OnRequest(UINT wFmt,HSZ hszTopic,HSZ hszItem)
{
  if(wFmt == CF_BITMAP)
     {
       return(hBitmapData[m_pMainWnd->nCurrentBitmap]); 
     }
  return NULL;
}

//***********************************************************
//
//  CSystemConv :: OnRequest()
//
//  Purpose: Passes system data to client
//
//  Parameters:  UINT wFmt
//               HSZ hszTopic
//               HSZ hszItem
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
HDDEDATA CMyConv::OnAdvReq(UINT wFmt,HSZ hszTopic,HSZ hszItem, UINT cAdvReq)
{
   
   if(wFmt == CF_BITMAP)
     {
       return(hBitmapData[m_pMainWnd->nCurrentBitmap]); 
     }
   return NULL;
}

//***********************************************************
//
//  CSystemConv :: OnAdvStart()
//
//  Purpose: Notification of start of advise loop
//
//  Parameters:  HSZ hszTopic
//               HSZ hszItem
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
BOOL CMyConv::OnAdvStart(HSZ hszTopic,HSZ hszItem) {return TRUE;}

//***********************************************************
//
//  CSystemConv :: OnAdvStop()
//
//  Purpose: Notification of stop of advise loop
//
//  Parameters:  HSZ hszTopic
//               HSZ hszItem
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMyConv::OnAdvStop(HSZ hszTopic,HSZ hszItem)  {}    

//***********************************************************
//
//  CMainWindow :: CMainWindow()
//
//  Purpose:  Constructor - Loads accelerator table and creates
//            main window 
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
CMainWindow::CMainWindow()
{
        LoadAccelTable( "MainAccelTable" );
       
                
        // load bitmaps
        Bitmaps[1].LoadBitmap("CIRCLE");
        Bitmaps[0].LoadBitmap("STREAMERS"); 
        Bitmaps[2].LoadBitmap("SHAPE");
        
        nCurrentBitmap = 2;
       
        Create( NULL, "CDDEServer Sample",
                WS_OVERLAPPEDWINDOW, rectDefault, NULL, "MainMenu" );
       
}

//***********************************************************
//
//  CMainWindow :: OnPaint()
//
//  Purpose:  Paints the main window.
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMainWindow::OnPaint()
{
        CPaintDC dc( this );
        CDC MemDC;
        MemDC.CreateCompatibleDC(&dc);
        CRect r;
        GetClientRect(&r);
        BITMAP bm;
        
       
        
        CBitmap *pBitmap = MemDC.SelectObject( &Bitmaps[nCurrentBitmap]);
        
        Bitmaps[nCurrentBitmap].GetObject(sizeof(BITMAP),&bm);
        dc.StretchBlt(0,0,r.Width(),r.Height(),&MemDC,0,0,
                       bm.bmWidth,bm.bmHeight,SRCCOPY);
                       
        MemDC.SelectObject(pBitmap);
}
 
//***********************************************************
//
//  CMainWindow :: OnAbout()
//
//  Purpose:  Brings up the about dialog box.
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMainWindow::OnAbout()
{
        CModalDialog about( "AboutBox", this );
        about.DoModal();
}
 
//***********************************************************
//
//  CMainWindow :: OnChangeBM()
//
//  Purpose:  Brings up the dialog box to change bitmap.
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMainWindow::OnChangeBM()
{
     CChangeBitmap ChangeDialog("SELECTDLG",this);
     
     ChangeDialog.nCurrentSelection = nCurrentBitmap;
     ChangeDialog.DoModal();
     nCurrentBitmap = ChangeDialog.nCurrentSelection; 
     
     // Update client rect
     Invalidate();
     
     (((CTheApp *)AfxGetApp())->m_pDDEServer)->DdePostAdvise( 
        (( (CTheApp *)AfxGetApp() ) ->m_pDDEServer)->hszTopics[BITMAP_TOPIC],
          CMyConv::hszItem);
}

//***********************************************************
//
//  CMainWindow :: OnDestroy()
//
//  Purpose:  Called when window is destroyed.
//          
//  Parameters:    
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CMainWindow::OnDestroy()
{
  CFrameWnd::OnDestroy();   
}

//***********************************************************
//
//  CMainWindow :: OnCreate()
//
//  Purpose:  Called when window is Created.
//          
//  Parameters: LPCREATESTRUCT lpc   
//
//  Return: (void)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
int CMainWindow::OnCreate(LPCREATESTRUCT lpc)
{  
    return 0;    
}

// CMainWindow message map:
BEGIN_MESSAGE_MAP( CMainWindow, CFrameWnd )
        ON_WM_PAINT()
        ON_COMMAND( IDM_ABOUT, OnAbout )
        ON_COMMAND( IDM_CHANGEBM, OnChangeBM )
        ON_WM_CREATE()
        ON_WM_DESTROY()
END_MESSAGE_MAP()

//***********************************************************
//
//  CTheApp :: InitInstance()
//
//  Purpose:  Initializes each instance of the application
//          
//  Parameters:  
//
//  Return: (BOOL)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
BOOL CTheApp::InitInstance()
{   
        m_pDDEServer = new CMyServer;

        m_pMainWnd = new CMainWindow();
        m_pMainWnd->ShowWindow( m_nCmdShow );
        m_pMainWnd->UpdateWindow();
        CMyConv::hszItem = 
             m_pDDEServer->DdeCreateStringHandle("ChangeBitmap");
        CMyConv::m_pMainWnd = (CMainWindow *)m_pMainWnd;

          for(int i = 0;i< NBITMAPS; ++i)
             {
             HBITMAP hBitmap =  (( ((CMainWindow *)m_pMainWnd) ->Bitmaps[i]).GetSafeHandle());
             CMyConv::hBitmapData[i] = m_pDDEServer->DdeCreateDataHandle(
                &hBitmap,
                sizeof(HBITMAP),
                0,
                CMyConv::hszItem,
                CF_BITMAP,
                HDATA_APPOWNED);
             }   
                       
        return TRUE;
}

//***********************************************************
//
//  CTheApp :: ExitInstance()
//
//  Purpose:  Cleans up after each instance of the application
//          
//  Parameters:  
//
//  Return: (int)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
int CTheApp::ExitInstance()
{
   m_pDDEServer->DdeFreeStringHandle( CMyConv::hszItem); 
   delete m_pDDEServer;
   return 0;
}

//***********************************************************
//
//  CChangeBitmap:: OnInitDialog
//
//  Purpose: Initializes dialogs controls
//          
//  Parameters:  
//
//  Return: (int)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
BOOL CChangeBitmap::OnInitDialog()
{
   CheckRadioButton(IDC_STREAMERS,IDC_SHAPES,IDC_STREAMERS+nCurrentSelection);
   return TRUE;
}

//***********************************************************
//
//  CChangeBitmap:: OnOK
//
//  Purpose: Saves current selections
//          
//  Parameters:  
//
//  Return: (int)
//
//  History:    Date       Author     Comment
//              09/21/92   BS         Created
//
//*************************************************************
void CChangeBitmap::OnOK()
{
   for(int i = IDC_STREAMERS;i<=IDC_SHAPES;++i)
      if(((CButton *)GetDlgItem(i))->GetCheck()==1)
         {
         nCurrentSelection = i-IDC_STREAMERS;
         break;
         }
         
    CModalDialog::OnOK();        
}


