//**********************************************************************
//  File name: server.h
//
//  Description: 
//            Declares the CMyServer,  CMainWindow, CMyConv, CSystemConv,
//             and CTheApp classes used by the SERVER application. 
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**********************************************************************

#include <afxwin.h>
#include "cddeserv.h"

//**********************************************************************
//
//  class CMyConv 
//
//  Purpose:
//  
//  Declares the conversation type used by the SERVER application. This 
//  is responsible getting the data to send to the clients.    
//
//  Derived from:  CDDEServerConv
// 
//  Data Members:  
//     static HSZ hszItem;             // item name 
//     static HDDEDATA hBitmapData[3]; // the bitmaps
//     static CMainWindow *m_pMainWnd; // window that displays the bitmap
//
//  Member Functions:
//
//    HDDEDATA OnRequest(UINT wFmt,HSZ hszTopic,HSZ hszItem);
//    HDDEDATA OnAdvReq(UINT wFmt,HSZ hszTopic,HSZ hszItem, UINT cAdvReq);
//    BOOL OnAdvStart(HSZ hszTopic,HSZ hszItem);
//    void OnAdvStop(HSZ hszTopic,HSZ hszItem);
//
//  Comments: 
//
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**************************************************************************

class CMainWindow;
class CMyConv : public CDDEServerConv
{
   public:
     static HSZ hszItem;
     static HDDEDATA hBitmapData[3];
     static CMainWindow *m_pMainWnd;
     
     
    // callbacks
    
    HDDEDATA OnRequest(UINT wFmt,HSZ hszTopic,HSZ hszItem);
    HDDEDATA OnAdvReq(UINT wFmt,HSZ hszTopic,HSZ hszItem, UINT cAdvReq);
    BOOL OnAdvStart(HSZ hszTopic,HSZ hszItem);
    void OnAdvStop(HSZ hszTopic,HSZ hszItem);
    
};

//**********************************************************************
//
//  class CSystemConv 
//
//  Purpose:
//  
//  Declares the conversation type used by the SERVER application. This 
//  getting the system data to send to the clients.    
//
//  Derived from:  CDDEServerConv
// 
//  Data Members:  
//     static HSZ hszItems[N_ITEMS];     // item names 
//     static HDDEDATA hData[N_ITEMS];   // data for each item
//
//  Member Functions:
//
//    static void Init();
//    static void CleanUp();  
//    HDDEDATA OnRequest(UINT wFmt,HSZ hszTopic,HSZ hszItem);
//
//  Comments: 
//
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**************************************************************************
#define N_ITEMS  7 
class CSystemConv : public CDDEServerConv
{
   public:
  
   static HSZ hszItems[N_ITEMS];
   static HDDEDATA hData[N_ITEMS];
   
   static void Init();
   static void CleanUp();
   
     
    // callbacks 
    
    HDDEDATA OnRequest( UINT wFmt, HSZ hszTopic,HSZ hszItem);
  
    
};

//**********************************************************************
//
//  class CMyServer
//
//  Purpose:
//     Creates appropriate conversation objects for each topic.     
//
//  Derived from:  CDDEServer
// 
//  Data Members:  
//     HSZ hszTopics[TOPICS];
//     HSZ hszServ;
//
//  Member Functions: 
//
//    Constructor
//     CMyServer();
//
//    Destructor 
//     ~CMyServer();
//
//     void Initialize();
//     void Uninitialize(); 
//     BOOL OnConnect(HSZ hszTopic,HSZ hszService,CONVCONTEXT FAR *pcc,
//            BOOL fSameInstance);
//     HDDEDATA OnWildConnect(UINT wFmt,HSZ hszTopic,HSZ hszService,
//            CONVCONTEXT FAR *pcc,BOOL fSameInstance);
//     void OnConnectConfirm(HCONV hConv,HSZ hszTopic,HSZ hszService,
//            BOOL fSameInstance); 
//
//  Comments: 
//
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**************************************************************************
#define TOPICS  2
#define  SYSTEM_TOPIC  0
#define BITMAP_TOPIC  1

class CMyServer : public CDDEServer
{
   public:
    
    CMyServer();
    ~CMyServer();
     
     HSZ hszTopics[TOPICS];
     HSZ hszServ;
     
     void Initialize();
     void Uninitialize();
     
     BOOL OnConnect(HSZ hszTopic,HSZ hszService,CONVCONTEXT FAR *pcc,BOOL fSameInstance);
     HDDEDATA OnWildConnect(UINT wFmt,HSZ hszTopic,HSZ hszService,CONVCONTEXT FAR *pcc,BOOL fSameInstance);
     void OnConnectConfirm(HCONV hConv,HSZ hszTopic,HSZ hszService,BOOL fSameInstance); 
};

//**********************************************************************
//
//  class CMainWindow 
//
//  Purpose:
//  
//  Declares main window class for the Server application.        
//
//  Derived from:  CFrameWnd
// 
//  Data Members:  
//      CBitmap Bitmaps[3];    // The three possible bitmaps
//      int nCurrentBitmap;    // The currently selected bitmap
//
//  Member Functions:
//
//  Constructor
//    CMainWindow();
//
//  Message handlers
//    afx_msg void OnPaint();
//    afx_msg void OnAbout();
//    afx_msg void OnDestroy();
//    afx_msg int OnCreate(LPCREATESTRUCT);
//    afx_msg void OnChangeBM();
//
//   Helper function
//    void GetSysConvData(LPSTR lpszMessage);
//
//  Comments: 
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**************************************************************************

#define NBITMAPS 3
class CMainWindow : public CFrameWnd
{
public:
   
        CBitmap Bitmaps[3];
        int nCurrentBitmap;
        
        CMainWindow();

        afx_msg void OnPaint();
        afx_msg void OnAbout();
        afx_msg void OnDestroy();
        afx_msg int OnCreate(LPCREATESTRUCT);
        afx_msg void OnChangeBM();
        
        
        // helper function for displaying output for system conversation
        void GetSysConvData(LPSTR lpszMessage);
        
        DECLARE_MESSAGE_MAP()
       

};

//**********************************************************************
//
//  class CTheApp 
//
//  Purpose:
//  
//  Declares application class for the Server application.        
//
//  Derived from:  CWinApp
// 
//  Data Members: 
//     CDDEClient *m_pDDEServer;     // pointer to DDE Server 
//
//  Member Functions:
// 
//     BOOL InitInstance();
//     int  ExitInstance();
//
//  Comments: 
//
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**************************************************************************

class CTheApp : public CWinApp
{
public:
        CMyServer *m_pDDEServer;
        
        BOOL InitInstance();
        int  ExitInstance();
};

//**********************************************************************
//
//  class CChangeBitmap 
//
//  Purpose:
//  
//  Dialog box for selecting the current bitmap        
//
//  Derived from:  CModalDialog
// 
//  Data Members: 
//     int nCurrentSelection;
//
//  Member Functions:
//    CChangeBitmap(LPSTR tempname,CWnd* pParent)
//    BOOL OnInitDialog();
//    void OnOK();
//
//  Comments: 
//
//
//  History:    Date       Author              Comment
//              09/21/92   Brian Scott         Created
//**************************************************************************


class CChangeBitmap : public CModalDialog
{
   public:
   
   int nCurrentSelection;
   
   CChangeBitmap(LPSTR tempname,CWnd* pParent) : CModalDialog(tempname,pParent) {}
   BOOL OnInitDialog();
   void OnOK();

}; 

