'* Input arrays
'*
'*   SortArray()
'*   TmpArray()
'*
'* The arrays must be made global to the sort routine.
'* Attempting to pass them as arguments to the QuickSort
'* routine would most likely result in an "Out of stack
'* space" error since the routine recursively calls itself.

' COPYRIGHT:
'
'   (C) Copyright Microsoft Corp. 1993.  All rights reserved.
'
'   You have a royalty-free right to use, modify, reproduce and
'   distribute the Sample Files (and/or any modified version) in
'   any way you find useful, provided that you agree that
'   Microsoft has no warranty obligations or liability for any
'   Sample Application Files which are modified.
'

DefInt A-Z

Sub QuickSort (Head, Tail)

    Dim HeadItem As IndexItemsType
    Dim CompItem As IndexItemsType
    Dim Item1 As IndexItemsType, Item2 As IndexItemsType

    If Head < Tail Then
            
            'Get the head element.  This element will be placed in
            'sorted position within the list.
            HeadItem = gSortArray(Head)
            

            'Set the index pointer to the next element after the head element
            CurrIndx = Head + 1
            
            'Assign temporary indexes
            TempHead = Head
            TempTail = Tail

            While TempHead < TempTail
                    
                    'Get the string to compare the head element with
                    CompItem = gSortArray(CurrIndx)
                    
                    If gSortOrder = 0 Then
                        Item1 = HeadItem   'Ascending order
                        Item2 = CompItem
                    Else
                        Item1 = CompItem   'Descending order
                        Item2 = HeadItem
                    End If
                    
                    
                    'If the HeadItem is greater, move the string being compared to the beginning of the array
                    'Note: This causes the elements to be sorted in ascending order.
                    '      To sort in descending order, change the greater- equal-than (>=)
                    '      comparison to a less-equal-than (<=) comparison.
                    If Cmp(Item1, Item2) >= 0 Then
                            gTmpArray(TempHead) = Item2
                            TempHead = TempHead + 1
                    Else

                            'If the head item is less, move the string being
                            'compared to the end of the array
                            gTmpArray(TempTail) = Item2
                            TempTail = TempTail - 1
                    End If

                    'Increment the index to make the next element of the array the
                    'current element to be sorted.
                    CurrIndx = CurrIndx + 1
            
            Wend

            'Insert the head item in sorted position in the list
            gTmpArray(TempHead) = HeadItem

            'Copy the sorted elements from the temporary array back to
            'the main array.
            For i = Head To Tail
                    gSortArray(i) = gTmpArray(i)
            Next i

            'Recursively call this procedure passing the first portion
            'of the array.
            Call QuickSort(Head, TempHead - 1)

            'Recursively call this procedure passing the last portion of
            'the array
            Call QuickSort(TempHead + 1, Tail)

    End If

End Sub

Function Cmp (Item1 As IndexItemsType, Item2 As IndexItemsType)
    
    'Territory is the primary key
    If UCase$(Item1.Territory) < UCase$(Item2.Territory) Then

        Cmp = -1
        Exit Function

    Else
        
        If UCase$(Item1.Territory) > UCase$(Item2.Territory) Then
            Cmp = 1
            Exit Function
        End If

    End If

    'Sales rep is sorted in ascending order within the territories
    If UCase$(Item1.SalesRep) < UCase$(Item2.SalesRep) Then

        Cmp = -1
        Exit Function

    Else
        
        If UCase$(Item1.SalesRep) > UCase$(Item2.SalesRep) Then
            Cmp = 1
            Exit Function
        End If

    End If

    'Report date is sorted in ascending order under sales rep
    If CmpDate(Item1.ReportDate, Item2.ReportDate) < 0 Then

        Cmp = -1
        Exit Function

    Else
        
        If CmpDate(Item1.ReportDate, Item2.ReportDate) > 0 Then

            Cmp = 1
            Exit Function

        Else

            Cmp = 0             'If the logic reaches this point, the records must be equal
            Exit Function

        End If

    End If
    

End Function

Function CmpDate (Item1 As Double, Item2 As Double)

    If Year(Item1) < Year(Item2) Then
        CmpDate = -1
        Exit Function
    Else
        If Year(Item1) > Year(Item2) Then
            CmpDate = 1
            Exit Function
        End If
    End If
    
    ' Continue if the year is equal

    If Month(Item1) < Month(Item2) Then
        CmpDate = -1
        Exit Function
    Else
        If Month(Item1) > Month(Item2) Then
            CmpDate = 1
            Exit Function
        End If
    End If
    
    ' Continue if the month is equal

    If Day(Item1) < Day(Item2) Then
        CmpDate = -1
        Exit Function
    Else
        If Day(Item1) > Day(Item2) Then
            CmpDate = 1
            Exit Function
        Else
            CmpDate = 0             'The dates must be equal
            Exit Function
        End If
    End If
    
End Function

