// COPYRIGHT:
//
//   (C) Copyright Microsoft Corp. 1993.  All rights reserved.
//
//   You have a royalty-free right to use, modify, reproduce and
//   distribute the Sample Files (and/or any modified version) in
//   any way you find useful, provided that you agree that
//   Microsoft has no warranty obligations or liability for any
//   Sample Application Files which are modified.
//
#include <afxwin.h>
#include "resource.h"
#include "streamer.h"


#include "wstream.h"
#include <iomanip.h>

//
// CMainWindow message map:
//
BEGIN_MESSAGE_MAP( CMainWindow, CFrameWnd )
	ON_WM_PAINT()
	
	ON_COMMAND( IDM_EXIT, OnExit )
	ON_COMMAND( IDM_ABOUT, OnAbout )
   ON_COMMAND( IDM_DISPLAY, OnDisplay ) 
		
	ON_COMMAND( IDM_WITH, OnWithTest )		
	ON_COMMAND( IDM_WITHOUT, OnWithOutTest )		
	ON_COMMAND( IDM_OUTPUT, OnOutputTest )		
	ON_COMMAND( IDM_VALUES, OnValuesTest )		
	ON_COMMAND( IDM_GET, OnGetTest )		
	ON_COMMAND( IDM_GETLINE, OnGetLineTest )		
	ON_COMMAND( IDM_DATE, OnDateTest)
	
END_MESSAGE_MAP()


//
// Menu Handlers
//
void CMainWindow::OnDisplay()
{
		wout << "This is a new line."  << endl;
}

void CMainWindow::OnGetTest()
{
   char line[25];
   wout << " [Get Test] Type a line terminated by enter or return\n>";
	win.get( line, 25 );
   wout << ' ' 
		<< line 
		<< "<---The first 25 characters entered." 
		<< endl;
}

void CMainWindow::OnGetLineTest()
{
   char line[25];
   wout << " [Get Line Test] Type a line terminated by enter or return\n>";
	win.getline( line, 25 );
   wout << ' ' 
		<< line 
		<< "<---The first 25 characters entered." 
		<< endl;
}

void CMainWindow::OnValuesTest()
{ 
	 int n[5] ;
	 int i ;
    wout << "Enter 5 values, separated by spaces" << endl;
    for( i = 0; i < 5; i++ ) {
        win >> n[i];
        if( win.eof() || win.bad() ) break; // Tests for end-of-file
                                            // or unrecoverable error
        if( win.fail() ) { // Tests for format conversion error
            win.clear(); // Clear stream's fail bit
            n[i] = 0;    // and continue processing
        }
    }
    for( i = 0; i < 5; i++ ) { // Prints the values just read
        wout << n[i];
    }
	 wout << "<----The Values" << endl ;
}

void CMainWindow::OnWithTest()
{
	seekgTest(TRUE) ;
}

void CMainWindow::OnWithOutTest()
{
	seekgTest(FALSE) ;
}

void CMainWindow::seekgTest( BOOL bWith )
{
   char line[100], ch = 0, *cp;

   cout << " Type a line terminated by 'x'\n>";
   cp = line;
   while ( ch != 'x' )
   {
      cin >> ch;
      if( !cin.good() ) break; // Exits on EOF or failure
      *cp++ = ch;
   }
   *cp = '\0';
   cout << ' ' << line;
	if( bWith )
	{
		win.seekg( 0L, ios::end ); // Empties the input stream
	}
   cout << "\n Type another one\n>";
   cp = line;
   ch = 0;
   while ( ch != 'x' )
   {
      cin.get( ch );
      if( !cin.good() ) break; // Exits on EOF or failure
      *cp++ = ch;
   }
   *cp = '\0';
   cout << ' ' << line << flush;

}

void CMainWindow::OnOutputTest()
{
	
   double values[] = { 1.23, 35.36, 653.7, 4358.24 };
   char *names[] = { "Zoot", "Jimmy", "Al", "Stan" };

	wout << "Testing various output stream operations.\n\n" ;
	
	for( int i = 0; i < 4; i++ )
      wout << values[i] << '\n';
	

	//exois102

	wout << endl
		<< "Testing width member functions"
		<< endl << endl ;
	
   for( i = 0; i < 4; i++ )
   {
      wout.width(10);
      wout << values[i] << '\n';
   }


	// exios103.cpp
		wout << endl 
			<< "Testing the setw member function"
			<< endl << endl;
	
   for( i = 0; i < 4; i++ )
      wout << setw( 6 )  << names[i]
           << setw( 10 ) << values[i] << endl;
		  
	// exios104.cpp
   wout << endl 
		<< "Testing the fill member function"
		<< endl << endl ;
	
	for( i = 0; i < 4; i++ )
   {
      wout.width( 10 );
      wout.fill( '*' );
      wout << values[i] << endl;
   }
	
	// Return fill character to normal
		wout.fill(' ') ;
	
	// exios105.cpp
	wout << endl 
		<< " Testing setiosflags and resetiosflags manipulators\n" 
		<< endl ;;
	
   for ( i = 0; i < 4; i++ )
      wout << setiosflags( ios::left )
           << setw( 6 )  << names[i]
           << resetiosflags( ios::left )
           << setw( 10 ) << values[i] << endl;
		  
		  
	// exios106.cpp
	wout << endl
		<< "Testing the setprecision manipulator"
		<< endl << endl ;
		
   for ( i = 0; i < 4; i++ )
      wout << setiosflags( ios::left )
           << setw( 6 )  
           << names[i]
           << resetiosflags( ios::left )
           << setw( 10 ) 
           << setprecision( 1 )
           << values[i] 
           << endl;		
		  
	// reset precision
		wout << setprecision(6) ;
	
	// exios107.cpp
	wout << endl 
		<< "Testing the fixed flag."
		<< endl << endl;

   wout << setiosflags( ios::fixed );
   for ( i = 0; i < 4; i++ )
      wout << setiosflags( ios::left )
           << setw( 6  )  
           << names[i]
           << resetiosflags( ios::left )
           << setw( 10 ) 
           << setprecision( 1 )
           << values[i] 
           << endl;	

	// reset precision
		wout << setprecision(6) << setw(6) ;
		  
}

//
// Date Class for the Date Input Test
//
class Date
{
   int mo, da, yr;
public:
   Date() {}
   friend ostream& operator<< ( ostream& os, Date& dt );
   friend istream& operator>> ( istream& is, Date& dt );
};

ostream& operator<< ( ostream& os, Date& dt )
{
   os << dt.mo << '/' << dt.da << '/' << dt.yr;
   return os;
}

istream& operator>> ( istream& is, Date& dt )
{
   is >> dt.mo >> dt.da >> dt.yr;
   return is;
}


void CMainWindow::OnDateTest()
{
   Date dt;
   wout << "Enter a date (mm dd yy): ";
   win >> dt;
   wout << dt << endl ;


}

//////////////////////////////////
// theApp:
// Just creating this application object runs the whole application.
//
CTheApp theApp;

//////////////////////////////////
//
// CMainWindow constructor:
//
CMainWindow::CMainWindow()
{
	LoadAccelTable( "MainAccelTable" );
	Create( NULL, "Streamer - wstreambuf Test Application",
		WS_OVERLAPPEDWINDOW, rectDefault, NULL, "MainMenu" );
}

//
// OnPaint:
//
void CMainWindow::OnPaint()
{
	CString s = "Streamer's Main Application Window!";
	CPaintDC dc( this );
	CRect rect;

	GetClientRect( rect );
	dc.SetTextAlign( TA_BASELINE | TA_CENTER );
	dc.SetTextColor( ::GetSysColor( COLOR_WINDOWTEXT ) );
	dc.SetBkMode(TRANSPARENT);
	dc.TextOut( ( rect.right / 2 ), ( rect.bottom / 2 ),
				s, s.GetLength() );
}

//
// OnAbout:
//
void CMainWindow::OnAbout()
{
	CModalDialog about( "AboutBox", this );
	about.DoModal();
}

//
// OnExit
//
void CMainWindow::OnExit()
{
	DestroyWindow() ;
}

/////////////////////////////////////////////////////////////////////////////
// CTheApp
//
//
BOOL CTheApp::InitInstance()
{
	m_pMainWnd = new CMainWindow();
	m_pMainWnd->ShowWindow( m_nCmdShow );
	m_pMainWnd->UpdateWindow();

	return TRUE;
}

CTheApp::~CTheApp()
{
	
}
