/*****************************************************************************/
/* Windows CPI-C Header file                                                 */
/*****************************************************************************/

#define CM_INT32 LONG
#define CM_ENTRY extern VOID WINAPI
#define CM_PTR   FAR *

typedef CM_INT32 CM_CONVERSATION_STATE;
typedef CM_INT32 CM_CONVERSATION_TYPE;
typedef CM_INT32 CM_DATA_RECEIVED_TYPE;
typedef CM_INT32 CM_DEALLOCATE_TYPE;
typedef CM_INT32 CM_ERROR_DIRECTION;
typedef CM_INT32 CM_FILL;
typedef CM_INT32 CM_PREPARE_TO_RECEIVE_TYPE;
typedef CM_INT32 CM_RECEIVE_TYPE;
typedef CM_INT32 CM_REQUEST_TO_SEND_RECEIVED;
typedef CM_INT32 CM_RETURN_CODE;
typedef CM_INT32 CM_RETURN_CONTROL;
typedef CM_INT32 CM_SEND_TYPE;
typedef CM_INT32 CM_STATUS_RECEIVED;
typedef CM_INT32 CM_SYNC_LEVEL;
typedef CM_INT32 CM_END_TYPE;

typedef CM_INT32 CM_CONVERSATION_SECURITY_TYPE;
typedef CM_INT32 XC_TP_NAME_TYPE;

/**STRUCT+********************************************************************/
/* Side information struct used by xcmssi to define side information         */
/*****************************************************************************/
typedef struct side_info_entry
{
  unsigned char   sym_dest_name[8];            /* symbolic destination name  */
  unsigned char   partner_LU_name[17];         /* fully Q'd PLU name         */
  unsigned char   reserved[3];                 /* future expansion ??        */
  XC_TP_NAME_TYPE TP_name_type;                /* set to XC_APPLICATION_TP   */
                                               /*  or to XC_SNA_SERVICE_TP   */
  unsigned char   TP_name[64];                 /* TP name                    */
  unsigned char   mode_name[8];                /* mode name                  */
  CM_CONVERSATION_SECURITY_TYPE
                  conversation_security_type;  /* set to CM_SECURITY_NONE    */
                                               /*  or to CM_SECURITY_SAME    */
                                               /*  or to CM_SECURITY_PROGRAM */
  unsigned char   security_user_ID[8];         /* user_ID                    */
  unsigned char   security_password[8];        /* password                   */
} SIDE_INFO;

/* conversation ID */
typedef char CONVERSATION_ID [8];

/**STRUCT-********************************************************************/


/*****************************************************************************/
/* Constant Definitions                                                      */
/*****************************************************************************/

/*****************************************************************************/
/* Values for the conversation_state                                         */
/*****************************************************************************/
typedef enum {
    CM_INITIALIZE_STATE            =    2,
    CM_SEND_STATE                  =    3,
    CM_RECEIVE_STATE               =    4,
    CM_SEND_PENDING_STATE          =    5,
    CM_CONFIRM_STATE               =    6,
    CM_CONFIRM_SEND_STATE          =    7,
    CM_CONFIRM_DEALLOCATE_STATE    =    8
} CONVERSATION_STATE;


/*****************************************************************************/
/* Values for the conversation_type                                          */
/*****************************************************************************/
typedef enum {
    CM_BASIC_CONVERSATION      =  0,
    CM_MAPPED_CONVERSATION     =  1
} CONVERSATION_TYPE;


/*****************************************************************************/
/* Values for data_received                                                  */
/*****************************************************************************/
typedef enum {
    CM_NO_DATA_RECEIVED          =    0,
    CM_DATA_RECEIVED             =    1,
    CM_COMPLETE_DATA_RECEIVED    =    2,
    CM_INCOMPLETE_DATA_RECEIVED  =    3
} DATA_RECEIVED;


/*****************************************************************************/
/* Values for the deallocate_type                                            */
/*****************************************************************************/
typedef enum {
    CM_DEALLOCATE_SYNC_LEVEL     =    0,
    CM_DEALLOCATE_FLUSH          =    1,
    CM_DEALLOCATE_CONFIRM        =    2,
    CM_DEALLOCATE_ABEND          =    3
} DEALLOCATE_TYPE;


/*****************************************************************************/
/* Values for the error_direction                                            */
/*****************************************************************************/
typedef enum {
    CM_RECEIVE_ERROR              =  0,
    CM_SEND_ERROR                 =  1
} ERROR_DIRECTION;


/*****************************************************************************/
/* Values for fill type                                                      */
/*****************************************************************************/
typedef enum {
    CM_FILL_LL                    =  0,
    CM_FILL_BUFFER                =  1
} CM_FILL;


/*****************************************************************************/
/* Values for prepare_to_receive_type                                        */
/*****************************************************************************/
typedef enum {
    CM_PREP_TO_RECEIVE_SYNC_LEVEL  =   0,
    CM_PREP_TO_RECEIVE_FLUSH       =   1,
    CM_PREP_TO_RECEIVE_CONFIRM     =   2
} PREPARE_TO_RECEIVE_TYPE;


/*****************************************************************************/
/* Values for the receive_type                                               */
/*****************************************************************************/
typedef enum {
    CM_RECEIVE_AND_WAIT          =    0,
    CM_RECEIVE_IMMEDIATE         =    1
} RECEIVE_TYPE;


/*****************************************************************************/
/* Values for request_to_send_received                                       */
/*****************************************************************************/
typedef enum {
    CM_REQ_TO_SEND_NOT_RECEIVED  =  0,
    CM_REQ_TO_SEND_RECEIVED      =  1
} REQUEST_TO_SEND_RECEIVED;

/*****************************************************************************/
/* Values for return_code                                                    */
/*****************************************************************************/
/*  return_code values  */
typedef enum {
    CM_OK                           = 0,
    CM_ALLOCATE_FAILURE_NO_RETRY    = 1,
    CM_ALLOCATE_FAILURE_RETRY       = 2,
    CM_CONVERSATION_TYPE_MISMATCH   = 3,
    CM_PIP_NOT_SPECIFIED_CORRECTLY  = 5,
    CM_SECURITY_NOT_VALID           = 6,
    CM_SYNC_LVL_NOT_SUPPORTED_LU    = 7,
    CM_SYNC_LVL_NOT_SUPPORTED_PGM   = 8,
    CM_TPN_NOT_RECOGNIZED           = 9,
    CM_TP_NOT_AVAILABLE_NO_RETRY    = 10,
    CM_TP_NOT_AVAILABLE_RETRY       = 11,
    CM_DEALLOCATED_ABEND            = 17,
    CM_DEALLOCATED_NORMAL           = 18,
    CM_PARAMETER_ERROR              = 19,
    CM_PRODUCT_SPECIFIC_ERROR       = 20,
    CM_PROGRAM_ERROR_NO_TRUNC       = 21,
    CM_PROGRAM_ERROR_PURGING        = 22,
    CM_PROGRAM_ERROR_TRUNC          = 23,
    CM_PROGRAM_PARAMETER_CHECK      = 24,
    CM_PROGRAM_STATE_CHECK          = 25,
    CM_RESOURCE_FAILURE_NO_RETRY    = 26,
    CM_RESOURCE_FAILURE_RETRY       = 27,
    CM_UNSUCCESSFUL                 = 28,
    CM_DEALLOCATED_ABEND_SVC        = 30,
    CM_DEALLOCATED_ABEND_TIMER      = 31,
    CM_SVC_ERROR_NO_TRUNC           = 32,
    CM_SVC_ERROR_PURGING            = 33,
    CM_SVC_ERROR_TRUNC              = 34,
    CM_OPERATION_INCOMPLETE         = 35,   /* not in SAA CPI-C  */
    CM_SYSTEM_EVENT                 = 36,   /* not in SAA CPI-C  */
    CM_OPERATION_NOT_ACCEPTED       = 37    /* SAA CPI-C Level 2 */
} CM_RETCODE;

/*****************************************************************************/
/* Values for return_control                                                 */
/*****************************************************************************/
typedef enum {
    CM_WHEN_SESSION_ALLOCATED    =  0,
    CM_IMMEDIATE                 =  1
} RETURN_CONTROL;



/*****************************************************************************/
/* Values for the send_type                                                  */
/*****************************************************************************/
typedef enum {
    CM_BUFFER_DATA                  = 0,
    CM_SEND_AND_FLUSH               = 1,
    CM_SEND_AND_CONFIRM             = 2,
    CM_SEND_AND_PREP_TO_RECEIVE     = 3,
    CM_SEND_AND_DEALLOCATE          = 4
} SEND_TYPE;



/*****************************************************************************/
/* Values for status_received                                                */
/*****************************************************************************/
typedef enum {
    CM_NO_STATUS_RECEIVED           = 0,
    CM_SEND_RECEIVED                = 1,
    CM_CONFIRM_RECEIVED             = 2,
    CM_CONFIRM_SEND_RECEIVED        = 3,
    CM_CONFIRM_DEALLOC_RECEIVED     = 4
} STATUS_RECEIVED;


/*****************************************************************************/
/* Values for the sync_level                                                 */
/*****************************************************************************/
typedef enum {
    CM_NONE                         = 0,
    CM_CONFIRM                      = 1
} SYNC_LEVEL;


/*****************************************************************************/
/* Values for the conversation_security_type (NS/2 compatible)               */
/*****************************************************************************/
typedef enum {
    CM_SECURITY_NONE                = 0,
    CM_SECURITY_SAME                = 1,
    CM_SECURITY_PROGRAM             = 2
} CONVERSATION_SECURITY_TYPE;


/* processing mode of conversation */
typedef enum {
    CM_BLOCKING                     = 0,
    CM_NON_BLOCKING                 = 1
} PROCESSING_MODE;


/*****************************************************************************/
/* Values for the TP_name_type (NS/2 compatible)                             */
/*****************************************************************************/
#define XC_APPLICATION_TP  (XC_TP_NAME_TYPE) 0
#define XC_SNA_SERVICE_TP  (XC_TP_NAME_TYPE) 1


/*****************************************************************************/
/* Values for end type (DCL addition)                                        */
/*****************************************************************************/
#define CM_HARD    (CM_INT32) 0
#define CM_SOFT    (CM_INT32) 1


/*****************************************************************************/
/* Function prototypes for CPIC call routines                                */
/*****************************************************************************/

/* Accept_Conversation */
CM_ENTRY cmaccp(unsigned char CM_PTR,		/* conversation_ID       */
                CM_INT32 CM_PTR);		/* return_code           */

/* Allocate */
CM_ENTRY cmallc(unsigned char CM_PTR,		/* conversation_ID       */
                CM_INT32 CM_PTR);		/* return_code           */

/* Cancel_Conversation */
CM_ENTRY cmcanc(unsigned char CM_PTR,		/* conversation_ID       */
                CM_INT32 CM_PTR);		/* return_code           */

/* Confirm */
CM_ENTRY cmcfm(unsigned char CM_PTR,		/* conversation_ID       */
               CM_INT32 CM_PTR,			/* request_to_send_rec.. */
               CM_INT32 CM_PTR);		/* return_code           */

/* Confirmed */
CM_ENTRY cmcfmd(unsigned char CM_PTR,		/* conversation_ID       */
                CM_INT32 CM_PTR);		/* return_code           */

/* Deallocate */
CM_ENTRY cmdeal(unsigned char CM_PTR,		/* conversation_ID       */
                CM_INT32 CM_PTR);		/* return_code           */

/* Extract_Conversation_State */
CM_ENTRY cmecs(unsigned char CM_PTR,		/* conversation_ID       */
               CM_INT32 CM_PTR,			/* conversation_state    */
               CM_INT32 CM_PTR);		/* return_code           */

/* Extract_Conversation_Type */
CM_ENTRY cmect(unsigned char CM_PTR,		/* conversation_ID       */
               CM_INT32 CM_PTR,			/* conversation type     */
               CM_INT32 CM_PTR);		/* return_code           */

/* Extract_Mode_Name */
CM_ENTRY cmemn(unsigned char CM_PTR,		/* conversation_ID       */
               unsigned char CM_PTR,		/* mode_name             */
               CM_INT32 CM_PTR,			/* mode_name_length      */
               CM_INT32 CM_PTR);		/* return_code           */

/* Extract_Partner_LU_Name */
CM_ENTRY cmepln(unsigned char CM_PTR,		/* conversation_ID       */
                unsigned char CM_PTR,		/* partner_LU_name       */
                CM_INT32 CM_PTR,		/* partner_LU_name_len.. */
                CM_INT32 CM_PTR);		/* return_code           */

/* Extract_Sync_Level */
CM_ENTRY cmesl(unsigned char CM_PTR,		/* conversation_ID       */
               CM_INT32 CM_PTR,			/* sync_level            */
               CM_INT32 CM_PTR);		/* return_code           */

/* Extract_TP_Name */
CM_ENTRY cmetpn(unsigned char CM_PTR,		/* conversation_ID       */
                unsigned char CM_PTR,		/* TP_name               */
                CM_INT32 CM_PTR,		/* TP_name_length        */
                CM_INT32 CM_PTR);		/* return_code           */

/* Flush */
CM_ENTRY cmflus(unsigned char CM_PTR,		/* conversation_ID       */
                CM_INT32 CM_PTR);		/* return_code           */

/* Initialize_Conversation */
CM_ENTRY cminit(unsigned char CM_PTR,		/* conversation_ID       */
                unsigned char CM_PTR,		/* sym_dest_name         */
                CM_INT32 CM_PTR);		/* return_code           */

/* Prepare_To_Receive */
CM_ENTRY cmptr(unsigned char CM_PTR,		/* conversation_ID       */
               CM_INT32 CM_PTR);		/* return_code           */

/* Receive */
CM_ENTRY cmrcv(unsigned char CM_PTR,		/* conversation_ID       */
               unsigned char CM_PTR,		/* buffer                */
               CM_INT32 CM_PTR,			/* requested_length      */
               CM_INT32 CM_PTR,			/* data_received         */
               CM_INT32 CM_PTR,			/* received_length       */
               CM_INT32 CM_PTR,			/* status_received       */
               CM_INT32 CM_PTR,			/* request_to_send_rec.. */
               CM_INT32 CM_PTR);		/* return_code           */

/* Request_To_Send */
CM_ENTRY cmrts(unsigned char CM_PTR,		/* conversation_ID       */
               CM_INT32 CM_PTR);		/* return_code           */

/* Send */
CM_ENTRY cmsend(unsigned char CM_PTR,		/* conversation_ID       */
                unsigned char CM_PTR,		/* buffer                */
                CM_INT32 CM_PTR,		/* send_length           */
                CM_INT32 CM_PTR,		/* request_to_send_rec.. */
                CM_INT32 CM_PTR);		/* return_code           */

/* Send_Error */
CM_ENTRY cmserr(unsigned char CM_PTR,		/* conversation_ID       */
                CM_INT32 CM_PTR,		/* request_to_send_rec.. */
                CM_INT32 CM_PTR);		/* return_code           */

/* Set_Conversation_Type */
CM_ENTRY cmsct(unsigned char CM_PTR,		/* conversation_ID       */
               CM_INT32 CM_PTR,			/* conversation_type     */
               CM_INT32 CM_PTR);		/* return_code           */

/* Set_Deallocate_Type */
CM_ENTRY cmsdt(unsigned char CM_PTR,		/* conversation_ID       */
               CM_INT32 CM_PTR,			/* deallocate_type       */
               CM_INT32 CM_PTR);		/* return_code           */

/* Set_Error_Direction */
CM_ENTRY cmsed(unsigned char CM_PTR,		/* conversation_ID       */
               CM_INT32 CM_PTR,			/* error_direction       */
               CM_INT32 CM_PTR);		/* return_code           */

/* Set_Fill */
CM_ENTRY cmsf(unsigned char CM_PTR,		/* conversation_ID       */
              CM_INT32 CM_PTR,			/* fill                  */
              CM_INT32 CM_PTR);			/* return_code           */

/* Set_Log_Data */
CM_ENTRY cmsld(unsigned char CM_PTR,		/* conversation_ID       */
               unsigned char CM_PTR,		/* log_data              */
               CM_INT32 CM_PTR,			/* log_data_length       */
               CM_INT32 CM_PTR);		/* return_code           */

/* Specify_Local _TP_Name */
CM_ENTRY cmsltp(unsigned char CM_PTR,		/* TP name               */
                CM_INT32 CM_PTR,		/* TP_name_length        */
                CM_INT32 CM_PTR);		/* return_code           */

/* Set_Mode_Name */
CM_ENTRY cmsmn(unsigned char CM_PTR,		/* conversation_ID       */
               unsigned char CM_PTR,		/* mode_name             */
               CM_INT32 CM_PTR,			/* mode_name_length      */
               CM_INT32 CM_PTR);		/* return_code           */

/* Set_Partner_LU_Name */
CM_ENTRY cmspln(unsigned char CM_PTR,		/* conversation_ID       */
                unsigned char CM_PTR,		/* partner_LU_name       */
                CM_INT32 CM_PTR,		/* partner_LU_name_len.. */
                CM_INT32 CM_PTR);		/* return_code           */

/* Set_Processing_Mode */
CM_ENTRY cmspm(unsigned char CM_PTR,		/* conversation_ID       */
               CM_INT32 CM_PTR,			/* processing mode       */
               CM_INT32 CM_PTR);		/* return_code           */

/* Set_Prepare_To_Receive_Type */
CM_ENTRY cmsptr(unsigned char CM_PTR,		/* conversation_ID       */
                CM_INT32 CM_PTR,		/* prepare_to_receive_.. */
                CM_INT32 CM_PTR);		/* return_code           */

/* Set_Receive_Type */
CM_ENTRY cmsrt(unsigned char CM_PTR,		/* conversation_ID       */
               CM_INT32 CM_PTR,			/* receive_type          */
               CM_INT32 CM_PTR);		/* return_code           */

/* Set_Return_Control */
CM_ENTRY cmsrc(unsigned char CM_PTR,		/* conversation_ID       */
               CM_INT32 CM_PTR,			/* return_control        */
               CM_INT32 CM_PTR);		/* return_code           */

/* Set_Send_Type */
CM_ENTRY cmsst(unsigned char CM_PTR,		/* conversation_ID       */
               CM_INT32 CM_PTR,			/* send_type             */
               CM_INT32 CM_PTR);		/* return_code           */

/* Set_Sync_Level */
CM_ENTRY cmssl(unsigned char CM_PTR,		/* conversation_ID       */
               CM_INT32 CM_PTR,			/* sync_level            */
               CM_INT32 CM_PTR);		/* return_code           */

/* Set_TP_name */
CM_ENTRY cmstpn(unsigned char CM_PTR,		/* conversation_ID       */
                unsigned char CM_PTR,		/* TP_name               */
                CM_INT32 CM_PTR,		/* TP_name_length         */
                CM_INT32 CM_PTR);		/* return_code           */

/* Test_Request_To_Send_Received */
CM_ENTRY cmtrts(unsigned char CM_PTR,		/* conversation_ID       */
                CM_INT32 CM_PTR,		/* request_to_send_rec.. */
                CM_INT32 CM_PTR);		/* return_code           */

/* Wait_For_Conversation */
CM_ENTRY cmwait(unsigned char CM_PTR,		/* conversation_ID       */
                CM_INT32 CM_PTR,		/* conversation ret code */
                CM_INT32 CM_PTR);		/* return_code           */

/* Set_Conversation_Security_User_ID */
CM_ENTRY cmscsu(unsigned char CM_PTR,		/* conversation_ID       */
                unsigned char CM_PTR,		/* user_ID               */
                CM_INT32 CM_PTR,		/* user_ID_length        */
                CM_INT32 CM_PTR);		/* return_code           */

/* Set_Conversation_Security_Password */
CM_ENTRY cmscsp(unsigned char CM_PTR,		/* conversation_ID       */
                unsigned char CM_PTR,		/* password              */
                CM_INT32 CM_PTR,		/* password_length       */
                CM_INT32 CM_PTR);		/* return_code           */

/*****************************************************************************/
/* Function Prototypes for additional NS/2 compatible call routines          */
/*****************************************************************************/

/* Delete_CPIC_Side_Information */
CM_ENTRY xcmdsi(unsigned char CM_PTR,		/* key_lock              */
                unsigned char CM_PTR,		/* sym_dest_name         */
                CM_INT32 CM_PTR);		/* return_code           */

/* Extract_CPIC_Side_Information */
CM_ENTRY xcmesi(CM_INT32 CM_PTR,		/* entry_number          */
                unsigned char CM_PTR,		/* sym_dest_name         */
                SIDE_INFO CM_PTR,		/* side_info_entry       */
                CM_INT32 CM_PTR,		/* side_info_length      */
                CM_INT32 CM_PTR);		/* return_code           */

/* Set_CPIC_Side_Information */
CM_ENTRY xcmssi(unsigned char CM_PTR,		/* key_lock              */
                SIDE_INFO CM_PTR,		/* side_info_entry       */
                CM_INT32 CM_PTR,		/* side_info length      */
                CM_INT32 CM_PTR);		/* return_code           */

/* Extract_Conversation_Security_Type */
CM_ENTRY xcecst(unsigned char CM_PTR,		/* conversation_ID       */
                CM_INT32 CM_PTR,		/* conversation_sec... */
                CM_INT32 CM_PTR);		/* return_code           */

/* Extract_Conversation_User_ID */
CM_ENTRY xcecsu(unsigned char CM_PTR,		/* conversation_ID       */
                unsigned char CM_PTR,		/* user_ID               */
                CM_INT32 CM_PTR,		/* user_ID_length        */
                CM_INT32 CM_PTR);		/* return_code           */

/* Set_Conversation_Security_Type */
CM_ENTRY xcscst(unsigned char CM_PTR,		/* conversation_ID       */
                CM_INT32 CM_PTR,		/* conversation_sec...	 */
                CM_INT32 CM_PTR);		/* return_code           */

/* Specify_Windows_Handle */
CM_ENTRY xchwnd(HWND,				/* Window HWND		 */
                CM_INT32 CM_PTR);		/* return_code           */

/*---------------------------------------------------------------------------*/
/* Windows CPI-C Extention Return Codes.                      		     */
/*---------------------------------------------------------------------------*/

#define WCPICALREADY         0xF000   /* An async call is already outstanding */
#define WCPICINVALID         0xF001   /* Async Task Id is invalid             */
#define WCPICCANCEL          0xF002   /* Blocking call was cancelled          */
#define WCPICSYSNOTREADY     0xF003   /* Underlying subsystem not started     */
#define WCPICVERNOTSUPPORTED 0xF004   /* Application version not supported    */


/*----------------------------------------------------------------------------*/
/* Api structre definitions                                                   */
/*----------------------------------------------------------------------------*/
#define	WCPICDESCRIPTION_LEN	127

typdef struct tagWCPICDATA {
	WORD	wVersion;
	char	szDescription[WCPICDESCRIPTION_LEN+1];
} WCPICDATA, * PWCPICDATA, FAR * LPWCPICDATA;

/*****************************************************************************/
/* ------------- Windows CPI-C function prototypes ------------------------- */
/*****************************************************************************/

extern BOOL WINAPI WinCPICCleanup(void);
extern BOOL WINAPI WinCPICIsBlocking(void);
extern int WINAPI WinCPICStartup(WORD, LPWCPICDATA );
extern FARPROC WINAPI WinCPICSetBlockingHook( FARPROC );
extern BOOL WINAPI WinCPICUnhookBlockingHook(void);


/*
 * These macros allow you to write programs that are easier to read, since
 * you can use the full name of the CPI-C call rather than its 6 character
 * entry point.
 *
 * When porting code that uses these macros, you will have to ensure that
 * the macros are defined on the target platform.
 */

#ifdef READABLE_MACROS

#define Accept_Conversation(v1,v2)            cmaccp(v1,v2)
#define Allocate(v1,v2)                       cmallc(v1,v2)
#define Cancel_Conversation(v1,v2)            cmcanc(v1,v2)
#define Confirm(v1,v2,v3)                     cmcfm(v1,v2,v3)
#define Confirmed(v1,v2)                      cmcfmd(v1,v2)
#define Deallocate(v1,v2)                     cmdeal(v1,v2)
#define Extract_Conversation_Security_User_ID(v1,v2,v3,v4)  cmecsu(v1,v2,v3,v4)
#define Extract_Conversation_Security_Type(v1,v2,v3) cmecst(v1,v2,v3)
#define Extract_Mode_Name(v1,v2,v3,v4)        cmemn(v1,v2,v3,v4)
#define Extract_Partner_LU_Name(v1,v2,v3,v4)  cmepln(v1,v2,v3,v4)
#define Extract_Sync_Level(v1,v2,v3)          cmesl(v1,v2,v3)
#define Extract_TP_Name(v1,v2,v3,v4)          cmetpn(v1,v2,v3,v4)
#define Flush(v1,v2)                          cmflus(v1,v2)
#define Initialize_Conversation(v1,v2,v3)     cminit(v1,v2,v3)
#define Prepare_To_Receive(v1,v2)             cmptr(v1,v2)
#define Receive(v1,v2,v3,v4,v5,v6,v7,v8)      cmrcv(v1,v2,v3,v4,v5,v6,v7,v8)
#define Request_To_Send(v1,v2)                cmrts(v1,v2)
#define Send_Error(v1,v2,v3)                  cmserr(v1,v2,v3)
#define Set_Conversation_Type(v1,v2,v3)       cmsct(v1,v2,v3)
#define Set_Deallocate_Type(v1,v2,v3)         cmsdt(v1,v2,v3)
#define Set_Error_Direction(v1,v2,v3)         cmsed(v1,v2,v3)
#define Send_Data(v1,v2,v3,v4,v5)             cmsend(v1,v2,v3,v4,v5)
#define Set_Fill(v1,v2,v3)                    cmsf(v1,v2,v3)
#define Set_Log_Data(v1,v2,v3,v4)             cmsld(v1,v2,v3,v4)
#define Set_Mode_Name(v1,v2,v3,v4)            cmsmn(v1,v2,v3,v4)
#define Set_Partner_LU_Name(v1,v2,v3,v4)      cmspln(v1,v2,v3,v4)
#define Set_Prepare_To_Receive_Type(v1,v2,v3) cmsptr(v1,v2,v3)
#define Set_Processing_Mode(v1,v2,v3)         cmspm(v1,v2,v3)
#define Set_Receive_Type(v1,v2,v3)            cmsrt(v1,v2,v3)
#define Set_Return_Control(v1,v2,v3)          cmsrc(v1,v2,v3)
#define Set_Send_Type(v1,v2,v3)               cmsst(v1,v2,v3)
#define Set_Sync_Level(v1,v2,v3)              cmssl(v1,v2,v3)
#define Set_TP_Name(v1,v2,v3,v4)              cmstpn(v1,v2,v3,v4)
#define Specify_Local_TP_Name(v1,v2,v3)       cmsltp(v1,v2,v3)
#define Set_Conversation_Security_Password(v1,v2,v3,v4) cmscsp(v1,v2,v3,v4)
#define Set_Conversation_Security_Type(v1,v2,v3) cmscst(v1,v2,v3)
#define Set_Conversation_Security_User_ID(v1,v2,v3,v4) cmscsu(v1,v2,v3,v4)
#define Test_Request_To_Send_Received(v1,v2,v3) cmtrts(v1,v2,v3)
#define Wait_For_Conversation(v1,v2,v3)       cmwait(v1,v2,v3)
#define Specify_Windows_Handle(v1,v2)         xchwnd(v1,v2)

#endif
