
/**********************************************************************/
/* wincsv.h                                                           */
/**********************************************************************/
/*       Function prototypes for the Common Service Verbs             */
/**********************************************************************/



/**********************************************************************/
/*                    VCB structures                                  */
/**********************************************************************/

typedef struct tagCSV {
	WORD	opcode;		/* Verb identifying operation code.     */
	BYTE	opext;		/* Reserved                             */
	BYTE	reserv2;	/* Reserved.                            */
	WORD	primary_rc;	/* Primary return code from verb.       */
	DWORD	secondary_rc;	/* Secondary (qualifying) return code.  */
} CSV, FAR * LPCSV;

typedef struct tagCONVERT {
	WORD	opcode;		/* Verb identifying operation code.     */
	BYTE	opext;		/* Reserved.                            */
	BYTE	reserv2;	/* Reserved.                            */
	WORD	primary_rc;	/* Primary return code from verb.       */
	DWORD	secondary_rc;	/* Secondary (qualifying) return code.  */
	BYTE	direction;	/* Direction of conversion - ASCII to   */
				/* EBCDIC or vice-versa.                */
	BYTE	char_set;	/* Character to use for the conversion  */
				/* A, AE, or user-defined G.            */
	WORD	len;		/* Length of string to be converted.    */
	LPBYTE	source;		/* Pointer to string to be converted.   */
	LPBYTE	target;		/* Address to put converted string at.  */
} CONVERT;



typedef struct tagGET_CP_CONVERT_TABLE {
	WORD	opcode;		/* Verb identifying operation code.    */
	BYTE	opext;		/* Reserved.                           */
	BYTE	reserv2;	/* Reserved.                           */
	WORD	primary_rc;	/* Primary return code from verb.      */
	DWORD	secondary_rc;	/* Secondary (qualifying) return code. */
	WORD	source_cp;	/* Src code page for conversion table. */
	WORD	target_cp;	/* Trgt code page for conversion table.*/
	LPBYTE	conv_tbl_addr;	/* Address to put conversion table at. */
	BYTE	char_not_fnd;	/* Character not found option: either  */
				/* substitute character or round trip. */
	BYTE	substitute_char;/* Substitute character to use.        */
} GET_CP_CONVERT_TABLE;




typedef struct tagTRANSFER_MS_DATA {
	WORD	opcode;		/* Verb identifying operation code.     */
	BYTE	data_type;	/* Data type supplied to the verb - can */
				/* be: user-defined; alert subvectors;  */
				/* pdstats subvectors; NMVT.            */
	BYTE	reserv2;	/* Reserved.                            */
	WORD	primary_rc;	/* Primary return code from verb.       */
	DWORD	secondary_rc;	/* Secondary (qualifying) return code.  */
	BYTE	options;	/* Verb options. Bits 3-7 are reserved. */
				/* Bit 0: Time stamp subvector option.  */
				/* Bit 1: Product set id subvector opt. */
				/* Bit 2: Option to send NMVT to SSCP.  */
				/* Bit 3: Option for local logging.     */
	BYTE	reserv3;	/* Reserved.                            */
	BYTE	origntr_id[8]	/* ID of originator of NMVT.            */
	WORD	dlen;		/* Length of data supplied to verb.     */
	LPBYTE	dptr;		/* Address of supplied data            .*/
} TRANSFER_MS_DATA;



/**********************************************************************/
/*        Verb op codes.                                              */
/**********************************************************************/

#define	SV_CONVERT                                 0x001A
#define	SV_GET_CP_CONVERT_TABLE                    0x0019
#define	SV_TRANSFER_MS_DATA                        0x001C


/**********************************************************************/
/*        Parameters for the Common Service Verbs.                    */
/**********************************************************************/

#define SV_ADD                                     0
#define SV_CHANGE                                  1
#define SV_IGNORE                                  0
#define SV_NO_ADD                                  1
#define SV_NO_SEND                                 1
#define SV_SEND                                    0

#define SV_A                                       0x01
#define SV_AE                                      0x00
#define SV_ALL                                     0x01
#define SV_ALERT_SUBVECTORS                        0x02
#define SV_ASCII_TO_EBCDIC                         0x00
#define SV_EBCDIC_TO_ASCII                         0x01
#define SV_FORWARD                                 0x02
#define SV_INTRV                                   0x00
#define SV_G                                       0x02
#define SV_LOG                                     0x00
#define SV_LOGS                                    0x00
#define SV_NMVT                                    0x01
#define SV_NO                                      0x00
#define SV_NO_INTRV                                0x01
#define SV_NO_LOG                                  0x01
#define SV_NONE                                    0x00
#define SV_OFF                                     0x00
#define SV_ON                                      0x01
#define SV_PDSTATS_SUBVECTORS                      0x03
#define SV_ROUND_TRIP                              0x01
#define SV_SOME                                    0x00
#define SV_SUBSTITUTE                              0x00
#define SV_SYSTEM_MESSAGES                         0x01
#define SV_USER_DEFINED                            0x00
#define SV_USER_MESSAGES                           0x02
#define SV_YES                                     0x01

/**********************************************************************/
/*       Bit masks for enabling and disabling functions               */
/**********************************************************************/
#define SV_BIT0                                    0x80   /* x....... */
#define SV_BIT1                                    0x40   /* .x...... */
#define SV_BIT2                                    0x20   /* ..x..... */
#define SV_BIT3                                    0x10   /* ...x.... */
#define SV_BIT4                                    0x08   /* ....x... */
#define SV_BIT5                                    0x04   /* .....x.. */
#define SV_BIT6                                    0x02   /* ......x. */
#define SV_BIT7                                    0x01   /* .......x */


/**********************************************************************/
/*        Return Codes from the Common Service Verbs.                 */
/**********************************************************************/

#define SV_COMM_SUBSYSTEM_NOT_LOADED               0x12F0
#define SV_CONFIG_FILE_ERROR                       0x3200
#define SV_INVALID_VERB                            0xFFFF
#define SV_INVALID_VERB_SEGMENT                    0x08F0
#define SV_OK                                      0x0000
#define SV_PARAMETER_CHECK                         0x0100
#define SV_STATE_CHECK                             0x0200
#define SV_UNEXPECTED_DOS_ERROR                    0x11F0

#define SV_CONVERSION_ERROR                        0x06040000
#define SV_DATA_EXCEEDS_RU_SIZE                    0x02030000
#define SV_INVALID_CHARACTER_SET                   0x02040000
#define SV_INVALID_CHAR_NOT_FOUND                  0x30060000
#define SV_INVALID_DATA_SEGMENT                    0x06000000
#define SV_INVALID_DATA_TYPE                       0x03030000
#define SV_INVALID_DIRECTION                       0x01040000
#define SV_INVALID_FIRST_CHARACTER                 0x04040000
#define SV_INVALID_FORWARD                         0x00070000
#define SV_INVALID_MESSAGE_ACTION                  0x21060000
#define SV_INVALID_NMVT_ERROR                      0x04030000
#define SV_INVALID_QUEUE                           0x04070000
#define SV_INVALID_RESET_TRACE                     0x2F060000
#define SV_INVALID_SELECTION                       0x02070000
#define SV_INVALID_SET                             0x24060000
#define SV_INVALID_SOURCE_CODE_PAGE                0x31060000
#define SV_INVALID_STORAGE_SIZE                    0x27060000
#define SV_INVALID_SUPPRESS                        0x01070000
#define SV_INVALID_TARGET_CODE_PAGE                0x32060000
#define SV_PD_MENUS_IN_PROGRESS                    0x2E060000
#define SV_QUEUE_ALREADY_IN_EFFECT                 0x05070000
#define SV_SSCP_PU_SESSION_NOT_ACTIVE              0x01030000
#define SV_TABLE_ERROR                             0x05040000
#define SV_TOO_HIGH_A_NUMBER                       0x03070000

/*--------------------------------------------------------------------*/
/* Windows CSV Extention Return Codes.                                */
/*--------------------------------------------------------------------*/

#define WCSVINVALID         0xF001 /* invalid  parameter              */
#define WCSVSYSNOTREADY     0xF003 /* Underlying subsystem not started*/
#define WCSVVERNOTSUPPORTED 0xF004 /* Appl'n version not supported    */

#define	 WCSVDESCRIPTION_LEN		127

typedef struct tagWCSVData {
	WORD  wVersion;
	char szDescription[WCSVDESCRIPTION_LEN+1];
} WCSVDATA, far *LPWCSVDATA;

extern VOID WINAPI WinCSV( LPCSV );
extern BOOL WINAPI WinCSVCleanup(void);
extern int WINAPI WinCSVStartup(WORD, LPWCSVDATA );