// COPYRIGHT:
//
//   (C) Copyright Microsoft Corp. 1993.  All rights reserved.
//
//   You have a royalty-free right to use, modify, reproduce and
//   distribute the Sample Files (and/or any modified version) in
//   any way you find useful, provided that you agree that
//   Microsoft has no warranty obligations or liability for any
//   Sample Application Files which are modified.
//

// wstream.cpp - Window output stream
//
// NOTE - This class is designed to work with large model only.  It may
// not work with medium or small model programs.
//
#include <afxwin.h>
#include "wio.h"
#include "wstream.h"

// Globals
wstreambuf wsb ;
istream_withassign win(&wsb) ;
ostream_withassign wout(&wsb) ;



// Automatically set up cin and cout to function like win and wout
class CInitialize
{
	public:
		CInitialize()
		{
			cin = &wsb ;
			cout = &wsb ;
		};
};

static CInitialize initializer ;

//
// Constructors
//
wstreambuf::wstreambuf( )
	: streambuf() 
{
	wioHandle = wioCreate(NULL) ;
}

wstreambuf::wstreambuf(char * title )
	: streambuf() 
{
	wioHandle = wioCreate(title) ;
}

wstreambuf::~wstreambuf()
{
	sync() ;
	wioDestroy(wioHandle) ;
}

int wstreambuf::sync() 
{
	TRACE("@@@--->sync\n") ;
	
	if (!unbuffered())
	{
		int out_count = out_waiting();	
		if( out_count )
		{
			wioDisplay(wioHandle,pbase(),out_count);
		}
		setp(0,0) ; //empty put area
		
		int in_count = in_avail() ;	
		if( in_count ) 
		{
			// Send chars back???
			wioReturnInput(wioHandle,gptr(),in_count) ;
		}
		setg(0,0,0) ; //empty get area
	}
	return 0 ;
}

int wstreambuf::overflow( int ch )
{
	TRACE("###--->Overflow\n") ;
	if( allocate()==EOF )
		return EOF ;
	if( wstreambuf::sync()==EOF ) // sync before new buffer created
		return EOF ;
	
	if (!unbuffered())
		setp(base(), base() + blen() ) ;	
			
	if( ch != EOF )
	{
		wioDisplay(wioHandle,(LPCSTR)&ch,1) ;
	}
	
	return 1 ;
}

//
// underflow
//
int wstreambuf::underflow()
{
	TRACE("***--->Underflow\r\n") ;
	if (in_avail())
		return (int)(unsigned char) *gptr() ;
	
	if (allocate()==EOF)
		return EOF ;
	
	if (wstreambuf::sync()==EOF)
		return EOF ;
	
	if( unbuffered() )
	{
		unsigned char tbuf ;
		if ( wioGetInput(wioHandle, (char*)&tbuf,1) )
		{
			return (int)tbuf ;
		}
		else
			return EOF ;
	} // unbuffered
		
	int count = wioGetInput(wioHandle, base(), blen()) ;
	if (count <= 0 )
		return EOF ;
	
	setg(base(), base(), base() + count) ;
	
	return (int)(unsigned char) *gptr() ;
}

//
// seekoff
//
streampos wstreambuf::seekoff(streamoff off,ios::seek_dir dir, int nMode)
{
	// only works for seeking the end
	// i.e. seekoff(0, ios::end, ios::in) ;

	if (  (nMode == ios::in)
		&& (dir == ios::end)
		&& (off == 0) )
	{
		wioFlushInput(wioHandle) ;
		return 0 ;
	}
	else
		return EOF ;
	
	
}

//
// pbackfail
//
int wstreambuf::pbackfail( int nCh )
{
	char ch = (char)nCh ;
	
	if (eback() < gptr())
		return sputbackc(ch) ;
	
	if (!wioReturnInput(wioHandle,&ch, 1))
		return EOF; 
	
	if(!unbuffered() && egptr())
	{
		memmove((gptr()+1),gptr(),(egptr()-(gptr()+1)));
		*gptr()=ch;
	}
	return nCh ;
}
